@@ -1,5 +1,5 @@
-/* $Id: compat.h,v 1.28 2005/09/10 16:48:01 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: compat.h,v 1.32 2006/06/19 20:28:52 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __compat_h_included
@@ -80,11 +80,14 @@
 #  define S_ISSOCK(fmode) 0
 #endif
 
+#if defined(IPPROTO_IPV6) && !defined(SOL_IPV6)
+#  define SOL_IPV6 IPPROTO_IPV6
+#endif
 
 /* all unsigned */
 #if !defined(HAVE_BASIC_SIZE_T) || !HAVE_BASIC_SIZE_T
 #  undef HAVE_BASIC_SIZE_T
-#  define HAVE_BASIC_SIZE_T 5
+#  define HAVE_BASIC_SIZE_T 6
 #endif
 #if HAVE_BASIC_SIZE_T==2
 #  define SIZET_MAX USHRT_MAX
@@ -526,6 +529,37 @@
 #  endif
 #endif
 
+
+/* at least for Linux */
+#define F_tv_sec "%ld"
+
+/* default: long */
+#if !defined(HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC) || !HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC
+#  undef HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC
+#  define HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC 5
+#endif
+#ifndef F_tv_usec
+#  if HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==1
+#define F_tv_usec "%06hd"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==2
+#define F_tv_usec "%06hu"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==3
+#define F_tv_usec "%06d"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==4
+#define F_tv_usec "%06u"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==5
+#define F_tv_usec "%06ld"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==6
+#define F_tv_usec "%06lu"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==7
+#define F_tv_usec "%06Ld"
+#  elif HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC==8
+#define F_tv_usec "%06Lu"
+#  else
+#error "HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC is out of range:" HAVE_TYPEOF_STRUCT_TIMEVAL_TV_USEC
+#  endif
+#endif
+
 /* default: long */
 #if !defined(HAVE_TYPEOF_RLIM_MAX) || !HAVE_TYPEOF_RLIM_MAX
 #  undef HAVE_TYPEOF_RLIM_MAX
@@ -573,4 +607,34 @@
 extern const char *hstrerror(int);
 #endif
 
+/*****************************************************************************/
+/* here are the declarations of compat.c */
+
+#if !HAVE_SIGACTION
+struct sigaction {
+   void (*sa_handler)(int);
+   void (*sa_sigaction)(int, siginfo_t *, void *);
+   sigset_t sa_mask;
+   int sa_flags;
+} ;
+struct siginfo {
+   int si_signo;
+   int si_errno;
+   int si_code;
+   pid_t si_pid;
+   uid_t si_uid;
+   int   si_status;
+   /*clock_t si_utime;*/
+   /*clock_t si_stime;*/
+   sigval_t si_value;
+   int      si_int;
+   void    *si_ptr;
+   void    *si_addr;
+   /*int      si_band;*/
+   /*int      si_fd;*/
+} ;
+extern int sigaction(int signum, const struct sigaction *act,
+		     struct sigaction *oldact);
+#endif /* !HAVE_SIGACTION */
+
 #endif /* !defined(__compat_h_included) */
