@@ -1,5 +1,5 @@
-/* $Id: filan.c,v 1.40 2006/07/13 06:32:57 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: filan.c,v 1.45 2007/02/08 19:42:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* the subroutine filan makes a "FILe descriptor ANalysis". It checks the
@@ -150,12 +150,12 @@ int filan_fd(int fd, FILE *outfile) {
 	       }
 	    }
 #endif /* defined(FIONREAD) */
-#if WITH_SOCKET
-	    if (isasocket(fd)) {
+#if WITH_SOCKET && defined(MSG_DONTWAIT)
+	    if ((ufds.revents & POLLIN) && isasocket(fd)) {
 	       char _peername[SOCKADDR_MAX];
 	       struct sockaddr *pa = (struct sockaddr *)_peername;
 	       struct msghdr msgh = {0};
-	       char peekbuff[0];
+	       char peekbuff[1];	/* [0] fails with some compilers */
 #if HAVE_STRUCT_IOVEC
 	       struct iovec iovec;
 #endif
@@ -180,13 +180,13 @@ int filan_fd(int fd, FILE *outfile) {
 #if HAVE_STRUCT_MSGHDR_MSGFLAGS
 	       msgh.msg_flags = 0;
 #endif
-	       if ((bytes = Recvmsg(fd, &msgh, MSG_PEEK)) < 0) {
+	       if ((bytes = Recvmsg(fd, &msgh, MSG_PEEK|MSG_DONTWAIT)) < 0) {
 		  Warn1("recvmsg(): %s", strerror(errno));
 	       } else {
 		  fprintf(outfile, "recvmsg="F_Zd", ", bytes);
 	       }
 	    }
-#endif /* WITH_SOCKET */
+#endif /* WITH_SOCKET && defined(MSG_DONTWAIT) */
 	 }	 
       }
    }
@@ -584,7 +584,7 @@ int sockan(int fd, FILE *outfile) {
       return -1;
    }
    fputc('\t', outfile);
-   fputs(sockaddr_info((struct sockaddr *)&sockname, nambuff, sizeof(nambuff)),
+   fputs(sockaddr_info((struct sockaddr *)&sockname, namelen, nambuff, sizeof(nambuff)),
 	 outfile);
 
    namelen = sizeof(peername);
@@ -596,7 +596,7 @@ int sockan(int fd, FILE *outfile) {
       /* only valid if getpeername() succeeded */
       fputs(" <-> ", outfile);
       fprintf(outfile, "%s\t",
-	      sockaddr_info((struct sockaddr *)&peername,
+	      sockaddr_info((struct sockaddr *)&peername, namelen,
 			    nambuff, sizeof(nambuff)));
    }
 
@@ -615,14 +615,15 @@ int sockan(int fd, FILE *outfile) {
       result = 0;
       break;
 #endif
-#if WITH_IP4 || WITH_IP6
 #if WITH_IP4
    case AF_INET:
+      result = ipan(fd, outfile);
+      break;
 #endif
 #if WITH_IP6
    case AF_INET6:
-#endif
       result = ipan(fd, outfile);
+      result |= ip6an(fd, outfile);
       break;
 #endif
    default:
@@ -712,6 +713,25 @@ int ipan(int fd, FILE *outfile) {
 #endif /* WITH_IP */
 
 
+#if WITH_IP6
+/* prints the option values for the IPv6 protocol */
+int ip6an(int fd, FILE *outfile) {
+   static const struct sockopt ip6opts[] = {
+#ifdef IPV6_V6ONLY
+      {IPV6_V6ONLY,         "IPV6_V6ONLY"},
+#endif
+      {0, NULL} } ;
+   const struct sockopt *optname;
+   
+   optname = ip6opts; while (optname->so) {
+      sockoptan(fd, optname, SOL_IPV6, outfile);
+      ++optname;
+   }
+   return 0;
+}
+#endif /* WITH_IP6 */
+
+
 #if WITH_TCP
 int tcpan(int fd, FILE *outfile) {
    static const struct sockopt tcpopts[] = {
@@ -826,13 +846,14 @@ int sockoptan(int fd, const struct sockopt *optname, int socklay, FILE *outfile)
 	     *(int *)optval, optlen);
       fprintf(outfile, "%s=%d\t", optname->name, *(int *)optval);
    } else {
-      char outbuf[512], *cp = outbuf;
+      char outbuf[FILAN_OPTLEN*9+128], *cp = outbuf;
       int i;
       for (i = 0; i < optlen/sizeof(unsigned int); ++i) {
 	 cp += sprintf(cp, "%08x ", ((unsigned int *)optval)[i]);
       }
       *--cp = '\0';	/* delete trailing space */
       Debug2("getsockopt(,,, {%s}, %d)", outbuf, optlen);
+      fflush(outfile);
       fprintf(outfile, "%s={%s}\t", optname->name, outbuf);
    }
    return 0;
