@@ -1,6 +1,6 @@
 #! /bin/sh
-# $Id: ftp.sh,v 1.10 2005/09/09 18:56:32 gerhard Exp $
-# Copyright Gerhard Rieger 2001-2005
+# $Id: ftp.sh,v 1.11 2006/12/28 07:27:01 gerhard Exp $
+# Copyright Gerhard Rieger 2001-2006
 # Published under the GNU General Public License V.2, see file COPYING
 
 # example how to write a shell script that communicates with stdio on the front
@@ -29,6 +29,7 @@ user="ftp"
 passwd="anonymous@domain.org"
 #method="socks4:socks"	# socks4 is address spec, socks is socks server name
 method=tcp
+addropts=
 
 # socat options
 SO1=
@@ -36,8 +37,21 @@ SO2=
 
 while :; do
     case "$1" in
-    -socks) shift; method="socks4:$1" ;;
-    -proxy) shift; method="proxy:$1" ;;
+    -socks|-socks4) shift;
+	case "$1" in
+	*:*) method="socks4:${1%%:*}"; addropts="socksport=${1#*:}" ;;
+	*) method="socks4:$1" ;;
+	esac ;;
+    -socks4a) shift;
+	case "$1" in
+	*:*) method="socks4a:${1%%:*}"; addropts="socksport=${1#*:}" ;;
+	*) method="socks4a:$1" ;;
+	esac ;;
+    -proxy) shift;
+	case "$1" in
+	*:*) method="proxy:${1%%:*}"; addropts="proxyport=${1#*:}" ;;
+	*) method="proxy:$1" ;;
+	esac ;;
     -user) shift; user="$1" ;;
     -passwd) shift; passwd="$1" ;;
     -t) set -vx ;;
@@ -56,7 +70,7 @@ export SO2
 server="$1"
 dir="$2"
 
-
+echo "addr=$method:$server:21,$addropts"; exit
 
 ### this is the central part to establish communication with socat ###
 ### copy these lines to make new communication shell scripts 
@@ -66,7 +80,7 @@ TMPDIR=$(if [ -x /bin/mktemp ]; then
 	    (umask 077; d=/tmp/$USER/FTPSH.$$; mkdir $d; echo $d)
 	 fi)
 TO="$TMPDIR/to"; FROM="$TMPDIR/from"
-socat $SO1 fifo:$TO,nonblock,ignoreeof!!fifo:$FROM $method:$server:21 &
+socat $SO1 fifo:$TO,nonblock,ignoreeof!!fifo:$FROM $method:$server:21,$addropts &
 S1=$!
 while ! [ -p "$TO" -a -p "$FROM" ]; do sleep 1; done
 exec 4>$TMPDIR/to 3<$TMPDIR/from
@@ -119,7 +133,7 @@ echo $info |tr ',' ' ' |(read i1 i2 i3 i4 p1 p2
 
     trap : 20
     # open data connection and transfer data
-    socat -u $SO2 $method:$server:$port -
+    socat -u $SO2 $method:$server:$port,$addropts -
 ) &
 S2=$!
 
