@@ -1,5 +1,5 @@
-/* $Id: socat.c,v 1.106 2006/07/13 21:16:04 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: socat.c,v 1.111 2007/03/06 21:03:28 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the main source, including command line option parsing, general
@@ -402,6 +402,11 @@ void socat_version(FILE *fd) {
 #else
    fputs("  #undef WITH_UNIX\n", fd);
 #endif /* WITH_UNIX */
+#ifdef WITH_ABSTRACT_UNIXSOCKET
+   fprintf(fd, "  #define WITH_ABSTRACT_UNIXSOCKET %d\n", WITH_ABSTRACT_UNIXSOCKET);
+#else
+   fputs("  #undef WITH_ABSTRACT_UNIXSOCKET\n", fd);
+#endif /* WITH_ABSTRACT_UNIXSOCKET */
 #ifdef WITH_IP4
    fprintf(fd, "  #define WITH_IP4 %d\n", WITH_IP4);
 #else
@@ -462,6 +467,11 @@ void socat_version(FILE *fd) {
 #else
    fputs("  #undef WITH_READLINE\n", fd);
 #endif
+#ifdef WITH_TUN
+   fprintf(fd, "  #define WITH_TUN %d\n", WITH_TUN);
+#else
+   fputs("  #undef WITH_TUN\n", fd);
+#endif
 #ifdef WITH_PTY
    fprintf(fd, "  #define WITH_PTY %d\n", WITH_PTY);
 #else
@@ -852,7 +862,7 @@ int _socat(void) {
 #endif
 	    return -1;
       } else if (retval == 0) {
-	 Info2("select timed out (no data within %d.%06u seconds)",
+	 Info2("select timed out (no data within %ld.%06ld seconds)",
 	       closing>=1?socat_opts.closwait.tv_sec:socat_opts.total_timeout.tv_sec,
 	       closing>=1?socat_opts.closwait.tv_usec:socat_opts.total_timeout.tv_usec);
 	 if (polling && !wasaction) {
@@ -903,6 +913,13 @@ int _socat(void) {
 	    maywr2 = false;
 	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
+	    /* is more data available that has already passed select()? */
+	    mayrd1 = (xiopending(sock1) > 0);
+	    if (XIO_RDSTREAM(sock1)->readbytes != 0 &&
+		XIO_RDSTREAM(sock1)->actbytes == 0) {
+	       /* avoid idle when all readbytes already there */
+	       mayrd1 = true;
+	    }          
 	 }
 	 /* (bytes1 == 0)  handled later */
       } else {
@@ -924,6 +941,13 @@ int _socat(void) {
 	    maywr1 = false;
 	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
+	    /* is more data available that has already passed select()? */
+	    mayrd2 = (xiopending(sock2) > 0);
+	    if (XIO_RDSTREAM(sock2)->readbytes != 0 &&
+		XIO_RDSTREAM(sock2)->actbytes == 0) {
+	       /* avoid idle when all readbytes already there */
+	       mayrd2 = true;
+	    }          
 	 }
 	 /* (bytes2 == 0)  handled later */
       } else {
@@ -1026,13 +1050,13 @@ static int
       return -1;
    }
    if (righttoleft) {
+      sprintf(buff, "%s%s length="F_Zu" from=%lu to=%lu\n",
+	      prefixrtol, timestamp, bytes, numrtol, numrtol+bytes-1);
       numrtol+=bytes;
-      sprintf(buff, "%s%s length="F_Zu" from=%lu to=%lu\n",
-	      prefixrtol, timestamp, bytes, numrtol, numrtol-1);
    } else {
-      numltor+=bytes;
       sprintf(buff, "%s%s length="F_Zu" from=%lu to=%lu\n",
-	      prefixltor, timestamp, bytes, numltor, numltor-1);
+	      prefixltor, timestamp, bytes, numltor, numltor+bytes-1);
+      numltor+=bytes;
    }
    fputs(buff, file);
    return 0;
@@ -1044,9 +1068,10 @@ static int
    buff should be at least twice as large as bufsiz, to allow all standard
    conversions. Returns the number of bytes written, or 0 on EOF or <0 if an
    error occurred or when data was read but none written due to conversions
-   (with EAGAIN). 
+   (with EAGAIN). EAGAIN also occurs when reading from a nonblocking FD where
+   the file has a mandatory lock.
    If 0 bytes were read (EOF), it does NOT shutdown or close a channel, and it
-   does NOT write 0 bytes.
+   does NOT write a zero bytes block.
    */
 /* inpipe, outpipe must be single descriptors (not dual!) */
 int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
@@ -1164,6 +1189,10 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 
 	    writt = xiowrite(outpipe, *buff, bytes);
 	    if (writt < 0) {
+	       /* EAGAIN when nonblocking but a mandatory lock is on file.
+		  the problem with EAGAIN is that the read cannot be repeated,
+		  so we need to buffer the data and try to write it later
+		  again. not yet implemented, sorry. */
 #if 0
 	       if (errno == EPIPE) {
 		  return 0;	/* can no longer write; handle like EOF */
