@@ -1,5 +1,5 @@
-/* $Id: sslcls.c,v 1.7 2006/06/30 20:46:32 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: sslcls.c,v 1.8 2007/02/26 21:30:58 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -221,6 +221,14 @@ int sycSSL_read(SSL *ssl, void *buf, int num) {
    return result;
 }
 
+int sycSSL_pending(SSL *ssl) {
+   int result;
+   Debug1("SSL_pending(%p)", ssl);
+   result = SSL_pending(ssl);
+   Debug1("SSL_pending() -> %d", result);
+   return result;
+}
+
 int sycSSL_write(SSL *ssl, const void *buf, int num) {
    int result;
    Debug3("SSL_write(%p, %p, %d)", ssl, buf, num);
