@@ -1,5 +1,5 @@
-/* $Id: sslcls.h,v 1.8 2006/06/30 20:46:37 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: sslcls.h,v 1.9 2007/02/26 21:30:58 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sslcls_h_included
@@ -35,6 +35,7 @@ int sycSSL_set_fd(SSL *ssl, int fd);
 int sycSSL_connect(SSL *ssl);
 int sycSSL_accept(SSL *ssl);
 int sycSSL_read(SSL *ssl, void *buf, int num);
+int sycSSL_pending(SSL *ssl);
 int sycSSL_write(SSL *ssl, const void *buf, int num);
 X509 *sycSSL_get_peer_certificate(SSL *ssl);
 int sycSSL_shutdown(SSL *ssl);
@@ -79,6 +80,7 @@ int sycFIPS_mode_set(int onoff);
 #define sycSSL_connect(s) SSL_connect(s)
 #define sycSSL_accept(s) SSL_accept(s)
 #define sycSSL_read(s,b,n) SSL_read(s,b,n)
+#define sycSSL_pending(s) SSL_pending(s)
 #define sycSSL_write(s,b,n) SSL_write(s,b,n)
 #define sycSSL_get_peer_certificate(s) SSL_get_peer_certificate(s)
 #define sycSSL_shutdown(s) SSL_shutdown(s)
