@@ -1,5 +1,5 @@
-/* $Id: sysutils.c,v 1.38 2006/05/31 19:14:19 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: sysutils.c,v 1.44 2007/03/06 21:05:11 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* translate socket addresses into human readable form */
@@ -13,6 +13,7 @@
 #include "mytypes.h"
 #include "error.h"
 #include "sycls.h"
+#include "utils.h"
 #include "sysutils.h"
 
 
@@ -100,7 +101,7 @@ void socket_in6_init(struct sockaddr_in6 *sa) {
 #endif /* WITH_IP6 */
 
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 /* initializes the socket address of the specified address family. Returns the
    length of the specific socket address, or 0 on error. */
 socklen_t socket_init(int af, union sockaddr_union *sa) {
@@ -114,17 +115,21 @@ socklen_t socket_init(int af, union sockaddr_union *sa) {
 #if WITH_IP6
    case AF_INET6:  socket_in6_init(&sa->ip6); return sizeof(sa->ip6);
 #endif
-   default: Error1("unknown address family %d", af);
+   default: Error1("socket_init(): unknown address family %d", af);
       memset(sa, 0, sizeof(union sockaddr_union));
       sa->soa.sa_family = af;
       return 0;
    }
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
+#if WITH_UNIX
+#define XIOUNIXSOCKOVERHEAD (sizeof(struct sockaddr_un)-sizeof(((struct sockaddr_un*)0)->sun_path))
+#endif
 
-#if WITH_SOCKET
-char *sockaddr_info(const struct sockaddr *sa, char *buff, size_t blen) {
+#if _WITH_SOCKET
+char *sockaddr_info(const struct sockaddr *sa, socklen_t salen, char *buff, size_t blen) {
+   char ubuff[5*UNIX_PATH_MAX+3];
    char *lbuff = buff;
    char *cp = lbuff;
    int n;
@@ -140,7 +145,30 @@ char *sockaddr_info(const struct sockaddr *sa, char *buff, size_t blen) {
 #if WITH_UNIX
    case 0:
    case AF_UNIX:
-      snprintf(cp, blen, "\"%s\"", (char *)&sa->sa_data);
+#if WITH_ABSTRACT_UNIXSOCKET
+      if (salen > XIOUNIXSOCKOVERHEAD &&
+	  sa->sa_data[0] == '\0') {
+	 char *nextc;
+//	 nextc =
+//	    sanitize_string((char *)&sa->sa_data+1, salen-XIOUNIXSOCKOVERHEAD-1,
+//			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+	 nextc =
+	    sanitize_string((char *)&sa->sa_data, salen-XIOUNIXSOCKOVERHEAD,
+			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+	 *nextc = '\0';
+//	 snprintf(cp, blen, "\"\\0%s\"", ubuff);
+	 snprintf(cp, blen, "\"%s\"", ubuff);
+      } else
+#endif /* WITH_ABSTRACT_UNIXSOCKET */
+      {
+	 char *nextc;
+	 nextc =
+	    sanitize_string((char *)&sa->sa_data,
+			    MIN(UNIX_PATH_MAX, strlen((char *)&sa->sa_data)),
+			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+	 *nextc = '\0';
+	 snprintf(cp, blen, "\"%s\"", ubuff);
+      }
       break;
 #endif
 #if WITH_IP4
@@ -166,11 +194,11 @@ char *sockaddr_info(const struct sockaddr *sa, char *buff, size_t blen) {
    }
    return lbuff;
 }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
 
 #if WITH_UNIX
-char *sockaddr_unix_info(const struct sockaddr_un *sa, char *buff, size_t blen) {
+char *sockaddr_unix_info(const struct sockaddr_un *sa, socklen_t salen, char *buff, size_t blen) {
    blen = Min(blen, sizeof(sa->sun_path));
    strncpy(buff, sa->sun_path, blen);
    if (strlen(buff) >= blen) {
@@ -181,6 +209,7 @@ char *sockaddr_unix_info(const struct sockaddr_un *sa, char *buff, size_t blen)
 #endif /* WITH_UNIX */
 
 #if WITH_IP4
+/* addr in host byte order! */
 char *inet4addr_info(uint32_t addr, char *buff, size_t blen) {
    if (snprintf(buff, blen, "%u.%u.%u.%u",
 		(unsigned int)(addr >> 24), (unsigned int)((addr >> 16) & 0xff),
@@ -207,6 +236,48 @@ char *sockaddr_inet4_info(const struct sockaddr_in *sa, char *buff, size_t blen)
 }
 #endif /* WITH_IP4 */
 
+#if !HAVE_INET_NTOP
+/* http://www.opengroup.org/onlinepubs/000095399/functions/inet_ntop.html */
+const char *inet_ntop(int pf, const void *binaddr,
+		      char *addrtext, socklen_t textlen) {
+   size_t retlen;
+   switch (pf) {
+   case PF_INET:
+      if ((retlen =
+	   snprintf(addrtext, textlen, "%u.%u.%u.%u",
+		    ((unsigned char *)binaddr)[0],
+		    ((unsigned char *)binaddr)[1],
+		    ((unsigned char *)binaddr)[2],
+		    ((unsigned char *)binaddr)[3]))
+	  < 0) {
+	 return NULL;	/* errno is valid */
+      }
+      break;
+   case PF_INET6:
+      if ((retlen =
+	   snprintf(addrtext, textlen, "%x:%x:%x:%x:%x:%x:%x:%x",
+		    ntohs(((uint16_t *)binaddr)[0]),
+		    ntohs(((uint16_t *)binaddr)[1]),
+		    ntohs(((uint16_t *)binaddr)[2]),
+		    ntohs(((uint16_t *)binaddr)[3]),
+		    ntohs(((uint16_t *)binaddr)[4]),
+		    ntohs(((uint16_t *)binaddr)[5]),
+		    ntohs(((uint16_t *)binaddr)[6]),
+		    ntohs(((uint16_t *)binaddr)[7])
+		    ))
+	  < 0) {
+	 return NULL;	/* errno is valid */
+      }
+      break;
+   default:
+      errno = EAFNOSUPPORT;
+      return NULL;
+   }
+   addrtext[retlen] = '\0';
+   return addrtext;
+}
+#endif /* !HAVE_INET_NTOP */
+
 #if WITH_IP6
 /* convert the IP6 socket address to human readable form. buff should be at
    least 50 chars long */
@@ -355,3 +426,70 @@ int parseport(const char *portname, int ipproto) {
    return se->s_port;
 }
 #endif /* WITH_TCP || WITH_UDP */
+
+#if WITH_IP4 || WITH_IP6
+/* check the systems interfaces for ifname and return its index
+   or -1 if no interface with this name was found */
+int ifindexbyname(const char *ifname) {
+   /* Linux: man 7 netdevice */
+   /* FreeBSD: man 4 networking */
+   /* Solaris: man 7 if_tcp */
+
+#if defined(HAVE_STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX)
+   /* currently we support Linux, FreeBSD; not Solaris */
+
+#define IFBUFSIZ 1024
+   int s;
+   struct ifreq ifr;
+
+   if (ifname[0] == '\0') {
+      return -1;
+   }
+   if ((s = Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
+      Error1("socket(PF_INET, SOCK_DGRAM, IPPROTO_IP): %s", strerror(errno));
+      return -1;
+   }
+
+   strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
+   if (Ioctl(s, SIOCGIFINDEX, &ifr) < 0) {
+      Close(s);
+      Info3("ioctl(%d, SIOCGIFINDEX, {%s}): %s",
+	     s, ifr.ifr_name, strerror(errno));
+      return -1;
+   }
+   Close(s);
+#if HAVE_STRUCT_IFREQ_IFR_INDEX
+   return ifr.ifr_index;
+#elif HAVE_STRUCT_IFREQ_IFR_IFINDEX
+   return ifr.ifr_ifindex;
+#endif /* HAVE_STRUCT_IFREQ_IFR_IFINDEX */
+
+#else /* !defined(HAVE_ STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX) */
+   return -1;
+#endif /* !defined(HAVE_ STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX) */
+}
+
+
+/* like ifindexbyname(), but allows an index number as input.
+   writes the resulting index to *ifindex and returns 0,
+   or returns -1 on error */
+int ifindex(const char *ifname, unsigned int *ifindex) {
+   char *endptr;
+   long int val;
+
+   if (ifname[0] == '\0') {
+      return -1;
+   }
+   val = strtol(ifname, &endptr, 0);
+   if (endptr[0] == '\0') {
+      *ifindex = val;
+      return 0;
+   }
+
+   if ((val = ifindexbyname(ifname)) < 0) {
+      return -1;
+   }
+   *ifindex = val;
+   return 0;
+}
+#endif /* WITH_IP4 || WITH_IP6 */
