@@ -1,5 +1,5 @@
-/* $Id: sysutils.h,v 1.18 2006/07/13 06:36:52 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: sysutils.h,v 1.23 2007/03/06 21:05:37 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sysutils_h_included
@@ -28,49 +28,49 @@ union sockaddr_union {
 #endif /* WITH_IP6 */
 } ;
 
-#if WITH_IP4
+#if _WITH_IP4
 struct xiorange_ip4 {
-   uint32_t netaddr;
-   uint32_t netmask;
+   struct in_addr netaddr;	/* network byte order */
+   struct in_addr netmask;	/* network byte order */
 } ;
-#endif /* WITH_IP4 */
+#endif /* _WITH_IP4 */
 
-#if WITH_IP6
+#if _WITH_IP6
 struct xiorange_ip6 {
    struct in6_addr addr;
    struct in6_addr mask;
 } ;
-#endif /* WITH_IP4 */
+#endif /* _WITH_IP4 */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 union xiorange_union {
-#if WITH_IP4
+#if _WITH_IP4
    struct xiorange_ip4 ip4;
-#endif /* WITH_IP4 */
-#if WITH_IP6
+#endif /* _WITH_IP4 */
+#if _WITH_IP6
    struct xiorange_ip6 ip6;
-#endif /* WITH_IP6 */
+#endif /* _WITH_IP6 */
 } ;
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 extern socklen_t socket_init(int af, union sockaddr_union *sa);
 #endif
 #if WITH_UNIX
 extern void socket_un_init(struct sockaddr_un *sa);
 #endif /* WITH_UNIX */
-#if WITH_IP4
+#if _WITH_IP4
 extern void socket_in_init(struct sockaddr_in *sa);
-#endif /* WITH_IP4 */
-#if WITH_IP6
+#endif /* _WITH_IP4 */
+#if _WITH_IP6
 extern void socket_in6_init(struct sockaddr_in6 *sa);
-#endif /* WITH_IP4 */
+#endif /* _WITH_IP4 */
 
-#if WITH_SOCKET
-extern char *sockaddr_info(const struct sockaddr *sa, char *buff, size_t blen);
+#if _WITH_SOCKET
+extern char *sockaddr_info(const struct sockaddr *sa, socklen_t salen, char *buff, size_t blen);
 #endif
 #if WITH_UNIX
-extern char *sockaddr_unix_info(const struct sockaddr_un *sa, char *buff, size_t blen);
+extern char *sockaddr_unix_info(const struct sockaddr_un *sa, socklen_t salen, char *buff, size_t blen);
 #endif /* WITH_UNIX */
 #if WITH_IP4
 extern char *inet4addr_info(uint32_t addr, char *buff, size_t blen);
@@ -79,6 +79,10 @@ extern char *sockaddr_inet4_info(const struct sockaddr_in *sa, char *buff, size_
 #if WITH_IP6
 extern char *sockaddr_inet6_info(const struct sockaddr_in6 *sa, char *buff, size_t blen);
 #endif /* WITH_IP6 */
+#if !HAVE_INET_NTOP
+extern const char *inet_ntop(int pf, const void *binaddr,
+			     char *addrtext, socklen_t textlen);
+#endif
 
 extern int getusergroups(const char *user, gid_t *list, size_t *ngroups);
 
@@ -88,4 +92,7 @@ extern const char *hstrerror(int err);
 
 extern int parseport(const char *portname, int proto);
 
+extern int ifindexbyname(const char *ifname);
+extern int ifindex(const char *ifname, unsigned int *ifindex);
+
 #endif /* !defined(__sysutils_h_included) */
