@@ -1,5 +1,5 @@
-/* $Id: xio-gopen.c,v 1.27 2006/05/31 19:27:55 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-gopen.c,v 1.32 2007/02/08 18:36:44 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of generic open type */
@@ -8,6 +8,7 @@
 #include "xioopen.h"
 
 #include "xio-named.h"
+#include "xio-unix.h"
 #include "xio-gopen.h"
 
 
@@ -47,8 +48,8 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
       int socktype = SOCK_STREAM;
       int optsotype = -1;
       struct sockaddr_un sa, us;
+      socklen_t salen, uslen = sizeof(us);
       bool needbind = false;
-      socklen_t uslen = sizeof(us);
       char infobuff[256];
       struct opt *opts2;
 
@@ -60,7 +61,7 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
 	 socktype = optsotype;
       }
 
-      if (retropt_bind(opts, AF_UNIX, socktype, 0, (struct sockaddr *)&us, &uslen, 0) >= 0) {
+      if (retropt_bind(opts, AF_UNIX, socktype, 0, (struct sockaddr *)&us, &uslen, 0, 0, 0) != STAT_NOACTION) {
 	 needbind = true;
       }
 
@@ -86,7 +87,7 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
       applyopts_cloexec(fd->stream.fd, opts);
 
       sa.sun_family = AF_UNIX;
-      strncpy(sa.sun_path, filename, sizeof(sa.sun_path));
+      salen = xiosetunix(&sa, filename, false, false);
 
 #if 0
       applyopts(fd->stream.fd, opts, PH_PREBIND);
@@ -107,14 +108,14 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
 #endif /* 0 */
 
       applyopts(fd->stream.fd, opts, PH_CONNECT);
-      if ((result = Connect(fd->stream.fd, (struct sockaddr *)&sa, sizeof(sa))) < 0) {
+      if ((result = Connect(fd->stream.fd, (struct sockaddr *)&sa, salen)) < 0) {
 	 if (errno == EINPROGRESS) {
 	    Warn4("connect(%d, %s, "F_Zd"): %s",
-		  fd->stream.fd, sockaddr_unix_info(&sa, infobuff, sizeof(infobuff)),
+		  fd->stream.fd, sockaddr_unix_info(&sa, salen, infobuff, sizeof(infobuff)),
 		  sizeof(sa), strerror(errno));
 	 } else if (errno == EPROTOTYPE && optsotype != SOCK_STREAM) {
 	    Warn4("connect(%d, %s, "F_Zd"): %s",
-		  fd->stream.fd, sockaddr_unix_info(&sa, infobuff, sizeof(infobuff)),
+		  fd->stream.fd, sockaddr_unix_info(&sa, salen, infobuff, sizeof(infobuff)),
 		  sizeof(sa), strerror(errno));
 	    Info("assuming datagram socket");
 	    Close(fd->stream.fd);
@@ -134,17 +135,19 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
 	    sa.sun_family = AF_UNIX;
 	    strncpy(sa.sun_path, filename, sizeof(sa.sun_path));
 	    
-	    fd->stream.dtype = DATA_RECVFROM;
+	    fd->stream.dtype = XIODATA_RECVFROM;
 	    fd->stream.salen = sizeof(sa);
 	    memcpy(&fd->stream.peersa.soa, &sa, fd->stream.salen);
 	 } else {
 	    Error4("connect(%d, %s, "F_Zd"): %s",
-		   fd->stream.fd, sockaddr_unix_info(&sa, infobuff, sizeof(infobuff)),
+		   fd->stream.fd, sockaddr_unix_info(&sa, fd->stream.salen, infobuff, sizeof(infobuff)),
 		   sizeof(sa), strerror(errno));
 	    return STAT_RETRYLATER;
 	 }
       }
-      fd->stream.howtoend = END_SHUTDOWN;
+      if (fd->stream.howtoend == END_UNSPEC) {
+	 fd->stream.howtoend = END_SHUTDOWN;
+      }
 
       applyopts_fchown(fd->stream.fd, opts);
       applyopts(fd->stream.fd, opts, PH_CONNECTED);
@@ -156,7 +159,7 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
 	       fd->stream.fd, &us, uslen, strerror(errno));
       } else {
 	 Notice1("successfully connected via %s",
-		 sockaddr_unix_info(&us, infobuff, sizeof(infobuff)));
+		 sockaddr_unix_info(&us, uslen, infobuff, sizeof(infobuff)));
       }
 #else
       Error("\"%s\" is a socket, but UNIX socket support is not compiled in");
@@ -180,7 +183,7 @@ static int xioopen_gopen(int argc, const char *argv[], struct opt *opts, int xio
 	      filetypenames[(st_mode&S_IFMT)>>12], filename, ddirection[(xioflags&XIO_ACCMODE)]);
       if ((result = _xioopen_open(filename, openflags, opts)) < 0)
 	 return result;
-#if __unix && __sun /* solaris */
+#ifdef I_PUSH
       if (S_ISCHR(st_mode)) {
 	 Ioctl(result, I_PUSH, "ptem");
 	 Ioctl(result, I_PUSH, "ldterm");
