@@ -1,12 +1,12 @@
-/* $Id: xio-ip.c,v 1.24 2006/07/13 06:41:09 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-ip.c,v 1.31 2007/03/06 21:08:02 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for IP related functions */
 
 #include "xiosysincludes.h"
 
-#if WITH_IP4 || WITH_IP6
+#if _WITH_IP4 || _WITH_IP6
 
 #include "xioopen.h"
 #include "xio-socket.h"
@@ -14,6 +14,8 @@
 #include "xio-ip6.h"
 
 
+#if WITH_IP4 || WITH_IP6
+
 #ifdef IP_OPTIONS
 const struct optdesc opt_ip_options = { "ip-options", "ipoptions", OPT_IP_OPTIONS, GROUP_SOCK_IP, PH_PASTSOCKET,TYPE_BIN, OFUNC_SOCKOPT_APPEND, SOL_IP, IP_OPTIONS };
 #endif
@@ -52,11 +54,16 @@ const struct optdesc opt_ip_freebind= { "ip-freebind","freebind",  OPT_IP_FREEBI
 #ifdef IP_ROUTER_ALERT
 const struct optdesc opt_ip_router_alert={"ip-router-alert","routeralert",OPT_IP_ROUTER_ALERT,GROUP_SOCK_IP,PH_PASTSOCKET,TYPE_INT,OFUNC_SOCKOPT,SOL_IP,IP_ROUTER_ALERT};
 #endif
-const struct optdesc opt_ip_multicast_ttl={"ip-multicast-ttl","multicastttl",OPT_IP_MULTICAST_TTL,GROUP_SOCK_IP,PH_PASTSOCKET,TYPE_INT,OFUNC_SOCKOPT,SOL_IP,IP_MULTICAST_TTL};
+/* following: Linux allows int but OpenBSD reqs char/byte */
+const struct optdesc opt_ip_multicast_ttl={"ip-multicast-ttl","multicastttl",OPT_IP_MULTICAST_TTL,GROUP_SOCK_IP,PH_PASTSOCKET,TYPE_BYTE,OFUNC_SOCKOPT,SOL_IP,IP_MULTICAST_TTL};
 const struct optdesc opt_ip_multicast_loop={"ip-multicast-loop","multicastloop",OPT_IP_MULTICAST_LOOP,GROUP_SOCK_IP,PH_PASTSOCKET,TYPE_INT,OFUNC_SOCKOPT,SOL_IP,IP_MULTICAST_LOOP};
+const struct optdesc opt_ip_multicast_if  ={"ip-multicast-if",  "multicast-if", OPT_IP_MULTICAST_IF,  GROUP_SOCK_IP,PH_PASTSOCKET,TYPE_IP4NAME,OFUNC_SOCKOPT,SOL_IP,IP_MULTICAST_IF};
 #ifdef IP_PKTOPTIONS
 const struct optdesc opt_ip_pktoptions = { "ip-pktoptions", "pktopts", OPT_IP_PKTOPTIONS, GROUP_SOCK_IP, PH_PASTSOCKET, TYPE_INT, OFUNC_SOCKOPT, SOL_IP, IP_PKTOPTIONS };
 #endif
+#ifdef IP_ADD_MEMBERSHIP
+const struct optdesc opt_ip_add_membership = { "ip-add-membership", "membership",OPT_IP_ADD_MEMBERSHIP, GROUP_SOCK_IP, PH_PASTSOCKET, TYPE_IP_MREQN, OFUNC_SOCKOPT, SOL_IP, IP_ADD_MEMBERSHIP };
+#endif
 
 #if HAVE_RESOLV_H
 const struct optdesc opt_res_debug    = { "res-debug",    NULL,       OPT_RES_DEBUG,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_DEBUG };
@@ -70,6 +77,9 @@ const struct optdesc opt_res_stayopen = { "res-stayopen", "stayopen", OPT_RES_ST
 const struct optdesc opt_res_dnsrch   = { "res-dnsrch",   "dnsrch",   OPT_RES_DNSRCH,   GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_DNSRCH };
 #endif /* HAVE_RESOLV_H */
 
+#endif /* WITH_IP4 || WITH_IP6 */
+
+
 #if HAVE_RESOLV_H
 int Res_init(void) {
    int result;
@@ -106,7 +116,7 @@ int xiogetaddrinfo(const char *node, const char *service,
    int port = -1;
    char *numnode = NULL;
    size_t nodelen;
-   unsigned long save_res_opts;
+   unsigned long save_res_opts = 0;
 #if HAVE_GETADDRINFO
    struct addrinfo hints = {0};
    struct addrinfo *res = NULL;
@@ -116,7 +126,7 @@ int xiogetaddrinfo(const char *node, const char *service,
    int error_num;
 
 #if HAVE_RESOLV_H
-   {
+   if (res_opts0 | res_opts1) {
       if (!(_res.options & RES_INIT)) {
          Res_init();	/*!!! returns -1 on error */
       }
@@ -157,8 +167,10 @@ int xiogetaddrinfo(const char *node, const char *service,
 	    this with AI_V4MAPPED, but not getaddrinfo() */
 	 if ((numnode = Malloc(strlen(node)+7+1)) == NULL) {
 #if HAVE_RESOLV_H
-	    _res.options = (_res.options & (~res_opts0&~res_opts1) |
-			    save_res_opts& ( res_opts0| res_opts1));
+	    if (res_opts0 | res_opts1) {
+	       _res.options = (_res.options & (~res_opts0&~res_opts1) |
+			       save_res_opts& ( res_opts0| res_opts1));
+	    }
 #endif
 	    return STAT_NORETRY;
 	 }
@@ -171,8 +183,10 @@ int xiogetaddrinfo(const char *node, const char *service,
    } else if (node && node[0] == '[' && node[(nodelen=strlen(node))-1]==']') {
       if ((numnode = Malloc(nodelen-1)) == NULL) {
 #if HAVE_RESOLV_H
-	 _res.options = (_res.options & (~res_opts0&~res_opts1) |
-			 save_res_opts& ( res_opts0| res_opts1));
+	 if (res_opts0 | res_opts1) {
+	    _res.options = (_res.options & (~res_opts0&~res_opts1) |
+			    save_res_opts& ( res_opts0| res_opts1));
+	 }
 #endif
 	 return STAT_NORETRY;
       }
@@ -209,8 +223,10 @@ int xiogetaddrinfo(const char *node, const char *service,
 	 if (numnode)  free(numnode);
 
 #if HAVE_RESOLV_H
-	 _res.options = (_res.options & (~res_opts0&~res_opts1) |
-			 save_res_opts& ( res_opts0| res_opts1));
+	 if (res_opts0 | res_opts1) {
+	    _res.options = (_res.options & (~res_opts0&~res_opts1) |
+			    save_res_opts& ( res_opts0| res_opts1));
+	 }
 #endif
 	 return STAT_RETRYLATER;
       }
@@ -346,8 +362,10 @@ int xiogetaddrinfo(const char *node, const char *service,
 		h_errno == NETDB_INTERNAL ? strerror(errno) :
 		hstrerror(h_errno));
 #if HAVE_RESOLV_H
-	 _res.options = (_res.options & (~res_opts0&~res_opts1) |
-			 save_res_opts& ( res_opts0| res_opts1));
+	 if (res_opts0 | res_opts1) {
+	    _res.options = (_res.options & (~res_opts0&~res_opts1) |
+			    save_res_opts& ( res_opts0| res_opts1));
+	 }
 #endif
 	 return STAT_RETRYLATER;
       }
@@ -395,17 +413,19 @@ int xiogetaddrinfo(const char *node, const char *service,
    if (numnode)  free(numnode);
 
 #if HAVE_RESOLV_H
-   _res.options = (_res.options & (~res_opts0&~res_opts1) |
-		   save_res_opts& ( res_opts0| res_opts1));
+   if (res_opts0 | res_opts1) {
+      _res.options = (_res.options & (~res_opts0&~res_opts1) |
+		      save_res_opts& ( res_opts0| res_opts1));
+   }
 #endif /* HAVE_RESOLV_H */
    return STAT_OK;
 }
 
 
-int parserange(char *rangename, int pf, union xiorange_union *range) {
+int xioparsenetwork(const char *rangename, int pf, union xiorange_union *range) {
 #if WITH_IP4
-   uint32_t *netaddr_in = &range->ip4.netaddr;
-   uint32_t *netmask_in = &range->ip4.netmask;
+   struct in_addr *netaddr_in = &range->ip4.netaddr;
+   struct in_addr *netmask_in = &range->ip4.netmask;
 #endif /* WITH_IP4 */
    struct hostent *maskaddr;
    char *delimpos;	/* absolute address of delimiter */
@@ -413,49 +433,78 @@ int parserange(char *rangename, int pf, union xiorange_union *range) {
 
   switch (pf) {
 #if WITH_IP4
+      char *rangename1;	/* a copy of rangename with writing allowed */
   case PF_INET:
-   if (delimpos = strchr(rangename, '/')) {
-      int i;
+      if ((rangename1 = strdup(rangename)) == NULL) {
+	 Error1("strdup(\"%s\"): out of memory", rangename);
+	 return STAT_RETRYLATER;
+      }
+
+   if (delimpos = strchr(rangename1, '/')) {
       bits = strtoul(delimpos+1, NULL, 10);
-      *netmask_in = 0xffffffff;
-      for (i = 32; i > bits; --i)
-         *netmask_in <<= 1;
-   } else if (delimpos = strchr(rangename, ':')) {
+      netmask_in->s_addr = htonl((0xffffffff << (32-bits)));
+   } else if (delimpos = strchr(rangename1, ':')) {
       if ((maskaddr = Gethostbyname(delimpos+1)) == NULL) {
 	 Error2("gethostbyname(\"%s\"): %s", delimpos+1,
 		h_errno == NETDB_INTERNAL ? strerror(errno) :
 		hstrerror(h_errno));
 	 return STAT_NORETRY;
       }
-      *netmask_in = *(unsigned long *)maskaddr->h_addr_list[0];
+      netmask_in->s_addr = *(uint32_t *)maskaddr->h_addr_list[0];
    } else {
-      Error1("parserange(\"%s\",,): missing netmask delimiter", rangename);
-      return -1;
+      Error1("xioparsenetwork(\"%s\",,): missing netmask delimiter", rangename);
+      free(rangename1);
+      return STAT_NORETRY;
    }
      {
 	struct hostent *nameaddr;
       *delimpos = 0;
-      if ((nameaddr = Gethostbyname(rangename)) == NULL) {
-	 Error2("gethostbyname(\"%s\"): %s", rangename,
+      if ((nameaddr = Gethostbyname(rangename1)) == NULL) {
+	 Error2("gethostbyname(\"%s\"): %s", rangename1,
 		h_errno == NETDB_INTERNAL ? strerror(errno) :
 		hstrerror(h_errno));
+	    free(rangename1);
 	 return STAT_NORETRY;
       }
-      *netaddr_in = ntohl(*(unsigned long *)nameaddr->h_addr_list[0]);
-      *netaddr_in &= *netmask_in;
+      netaddr_in->s_addr = *(unsigned long *)nameaddr->h_addr_list[0];
      }
+      free(rangename1);
    break;
 #endif /* WITH_IP4 */
 #if WITH_IP6
    case PF_INET6:
-      return xioparserange_ip6(rangename, &range->ip6);
+      return xioparsenetwork_ip6(rangename, &range->ip6);
       break;
 #endif /* WITH_IP6 */
   default:
      Error1("range option not supported with address family %d", pf);
      return STAT_NORETRY;
   }
-  return 0;
+  return STAT_OK;
 }
 
-#endif /* WITH_IP4 || WITH_IP6 */
+/* parses a string of form address/bits or address:mask, and fills the fields 
+   of the range union. The addr component is masked with mask. */
+int parserange(const char *rangename, int pf, union xiorange_union *range) {
+   if (xioparsenetwork(rangename, pf, range) < 0) {
+      return -1;
+   }
+   switch (pf) {
+#if WITH_IP4
+   case PF_INET:
+      range->ip4.netaddr.s_addr &= range->ip4.netmask.s_addr;
+      break;
+#endif /* WITH_IP4 */
+#if WITH_IP6
+   case PF_INET6:
+      return xiorange_ip6andmask(&range->ip6);
+      break;
+#endif /* WITH_IP6 */
+   default:
+      Error1("range option not supported with address family %d", pf);
+      return STAT_NORETRY;
+   }
+   return 0;
+}
+
+#endif /* _WITH_IP4 || _WITH_IP6 */
