@@ -1,5 +1,5 @@
-/* $Id: xio-ip4.c,v 1.10 2006/07/13 06:41:43 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-ip4.c,v 1.13 2007/03/06 21:08:38 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for IP4 related functions */
@@ -16,8 +16,8 @@
 /* check if peer address is within permitted range.
    return >= 0 if so. */
 int xiocheckrange_ip4(struct sockaddr_in *pa, struct xiorange_ip4 *range) {
-   uint32_t *netaddr_in = &range->netaddr;
-   uint32_t *netmask_in = &range->netmask;
+   struct in_addr *netaddr_in = &range->netaddr;
+   struct in_addr *netmask_in = &range->netmask;
    char addrbuf[256], maskbuf[256];
    char peername[256];
 
@@ -28,15 +28,14 @@ int xiocheckrange_ip4(struct sockaddr_in *pa, struct xiorange_ip4 *range) {
    }
    /* client address restriction */
    Debug2("permitted client subnet: %s:%s",
-	  inet4addr_info(*netaddr_in, addrbuf, sizeof(addrbuf)),
-	  inet4addr_info(*netmask_in, maskbuf, sizeof(maskbuf)));
+	  inet4addr_info(ntohl(netaddr_in->s_addr), addrbuf, sizeof(addrbuf)),
+	  inet4addr_info(ntohl(netmask_in->s_addr), maskbuf, sizeof(maskbuf)));
    Debug1("client address is 0x%08x",
 	  ntohl(pa->sin_addr.s_addr));
    Debug1("masked address is 0x%08x",
-	  ntohl(pa->sin_addr.s_addr) &
-	  *netmask_in);
-   if ((ntohl(pa->sin_addr.s_addr) & *netmask_in)
-       != *netaddr_in) {
+	  ntohl(pa->sin_addr.s_addr & netmask_in->s_addr));
+   if ((pa->sin_addr.s_addr & netmask_in->s_addr)
+       != netaddr_in->s_addr) {
       Debug1("client address %s is not permitted",
 	     sockaddr_inet4_info(pa, peername, sizeof(peername)));
       return -1;
