@@ -1,5 +1,5 @@
-/* $Id: xio-ip6.c,v 1.28 2006/07/13 06:42:15 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-ip6.c,v 1.31 2007/03/06 21:19:18 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for IP6 related functions */
@@ -17,8 +17,11 @@
 #ifdef IPV6_V6ONLY
 const struct optdesc opt_ipv6_v6only = { "ipv6-v6only", "ipv6only", OPT_IPV6_V6ONLY, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_BOOL, OFUNC_SOCKOPT, SOL_IPV6, IPV6_V6ONLY };
 #endif
+#ifdef IPV6_JOIN_GROUP
+const struct optdesc opt_ipv6_join_group = { "ipv6-join-group", "join-group", OPT_IPV6_JOIN_GROUP, GROUP_SOCK_IP6, PH_PASTBIND, TYPE_IP_MREQN, OFUNC_SOCKOPT, SOL_IPV6, IPV6_JOIN_GROUP };
+#endif
 
-int xioparserange_ip6(char *rangename, struct xiorange_ip6 *range) {
+int xioparsenetwork_ip6(const char *rangename, struct xiorange_ip6 *range) {
    char *delimpos;	/* absolute address of delimiter */
    size_t delimind;	/* index of delimiter in string */
    int bits;
@@ -35,13 +38,14 @@ int xioparserange_ip6(char *rangename, struct xiorange_ip6 *range) {
       return STAT_NORETRY;
    }
    if ((delimpos = strchr(rangename, '/')) == NULL) {
-      Error1("xioparserange_ip6(\"%s\",,): missing mask bits delimiter '/'",
+      Error1("xioparsenetwork_ip6(\"%s\",,): missing mask bits delimiter '/'",
 	     rangename);
       return STAT_NORETRY;
    }
    delimind = delimpos - rangename;
 
    if ((baseaddr = strdup(rangename+1)) == NULL) {
+      Error1("strdup(\"%s\"): out of memory", rangename+1);
       return STAT_NORETRY;
    }
    baseaddr[delimind-1] = '\0';
@@ -83,6 +87,21 @@ int xioparserange_ip6(char *rangename, struct xiorange_ip6 *range) {
    return 0;
 }
 
+int xiorange_ip6andmask(struct xiorange_ip6 *range) {
+   int i;
+#if 0
+   range->addr.s6_addr32[0] &= range->mask.s6_addr32[0];
+   range->addr.s6_addr32[1] &= range->mask.s6_addr32[1];
+   range->addr.s6_addr32[2] &= range->mask.s6_addr32[2];
+   range->addr.s6_addr32[3] &= range->mask.s6_addr32[3];
+#else
+   for (i = 0; i < 16; ++i) {
+      range->addr.s6_addr[i] &= range->mask.s6_addr[i];
+   }
+#endif
+   return 0;
+}
+
 /* check if peer address is within permitted range.
    return >= 0 if so. */
 int xiocheckrange_ip6(struct sockaddr_in6 *pa, struct xiorange_ip6 *range) {
