@@ -1,5 +1,5 @@
-/* $Id: xio-ip6.h,v 1.10 2006/06/19 20:29:39 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-ip6.h,v 1.13 2007/03/06 21:19:18 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_ip6_h_included
@@ -8,9 +8,11 @@
 #if WITH_IP6
 
 extern const struct optdesc opt_ipv6_v6only;
+extern const struct optdesc opt_ipv6_join_group;
 
 extern
-int xioparserange_ip6(char *rangename, struct xiorange_ip6 *range);
+int xioparsenetwork_ip6(const char *rangename, struct xiorange_ip6 *range);
+extern int xiorange_ip6andmask(struct xiorange_ip6 *range);
 
 extern
 int xiocheckrange_ip6(struct sockaddr_in6 *pa, struct xiorange_ip6 *range);
