@@ -1,5 +1,5 @@
-/* $Id: xio-ipapp.c,v 1.31 2006/07/13 06:42:37 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-ipapp.c,v 1.34 2007/02/08 18:27:00 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for TCP and UDP related options */
@@ -42,6 +42,8 @@ int xioopen_ipapp_connect(int argc, const char *argv[], struct opt *opts,
       Error2("%s: wrong number of parameters (%d instead of 2)", argv[0], argc-1);
    }
 
+   xfd->howtoend = END_SHUTDOWN;
+
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
    applyopts(-1, opts, PH_INIT);
 
@@ -172,13 +174,12 @@ int
       *pf = them->soa.sa_family;
    }
 
-   applyopts(-1, opts, PH_INIT);
-
    applyopts(-1, opts, PH_EARLY);
 
    /* 3 means: IP address AND port accepted */
-   if (retropt_bind(opts, *pf, *socktype, protocol, (struct sockaddr *)us, uslen, 3)
-       >= 0) {
+   if (retropt_bind(opts, *pf, *socktype, protocol, (struct sockaddr *)us, uslen, 3,
+		    res_opts0, res_opts1)
+       != STAT_NOACTION) {
       *needbind = true;
    } else {
       switch (*pf) {
@@ -210,7 +211,7 @@ int
    *opts0 = copyopts(opts, GROUP_ALL);
 
    Notice1("opening connection to %s",
-	   sockaddr_info((struct sockaddr *)them, infobuff, sizeof(infobuff)));
+	   sockaddr_info((struct sockaddr *)them, *themlen, infobuff, sizeof(infobuff)));
    return STAT_OK;
 }
 #endif /* WITH_IP4 */
@@ -270,6 +271,8 @@ int xioopen_ipapp_listen(int argc, const char *argv[], struct opt *opts,
 #endif
    }
 
+   fd->stream.howtoend = END_SHUTDOWN;
+
    if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
    applyopts(-1, opts, PH_INIT);
    applyopts(-1, opts, PH_EARLY);
