@@ -1,5 +1,5 @@
-/* $Id: xio-listen.c,v 1.41 2006/07/13 06:43:32 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-listen.c,v 1.44 2007/02/08 18:27:00 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for listen socket options */
@@ -96,6 +96,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    struct sockaddr sa;
    socklen_t salen;
    int backlog = 5;	/* why? 1 seems to cause problems under some load */
+   char *rangename;
    bool dofork = false;
    pid_t pid;	/* mostly int; only used with fork */
    char infobuff[256];
@@ -151,7 +152,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    applyopts(xfd->fd, opts, PH_BIND);
    if (Bind(xfd->fd, (struct sockaddr *)us, uslen) < 0) {
       Msg4(level, "bind(%d, {%s}, "F_Zd"): %s", xfd->fd,
-	   sockaddr_info(us, infobuff, sizeof(infobuff)), uslen,
+	   sockaddr_info(us, uslen, infobuff, sizeof(infobuff)), uslen,
 	   strerror(errno));
       Close(xfd->fd);
       return STAT_RETRYLATER;
@@ -186,11 +187,13 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    }
 
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &xfd->para.socket.rangename) >= 0) {
-      if (parserange(xfd->para.socket.rangename, us->sa_family,
-		     &xfd->para.socket.range) < 0) {
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, us->sa_family, &xfd->para.socket.range) < 0) {
+	 free(rangename);
 	 return STAT_NORETRY;
       }
+      free(rangename);
+      xfd->para.socket.dorange = true;
    }
 #endif
 
@@ -215,9 +218,9 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
       char peername[256];
       char sockname[256];
       int ps;		/* peer socket */
-      char _peername[SOCKADDR_MAX];
+      union sockaddr_union _peername;
       union sockaddr_union _sockname;
-      union sockaddr_union *pa = (union sockaddr_union *)_peername;	/* peer address */
+      union sockaddr_union *pa = &_peername;	/* peer address */
       union sockaddr_union *la = &_sockname;	/* local address */
       socklen_t pas = sizeof(_peername);	/* peer address size */
       socklen_t las = sizeof(_sockname);	/* local address size */
@@ -225,7 +228,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 
       do {
 	 /*? int level = E_ERROR;*/
-	 Notice1("listening on %s", sockaddr_info(us, lisname, sizeof(lisname)));
+	 Notice1("listening on %s", sockaddr_info(us, uslen, lisname, sizeof(lisname)));
 	 ps = Accept(xfd->fd, (struct sockaddr *)&sa, &salen);
 	 if (ps >= 0) {
 	    /*0 Info4("accept(%d, %p, {"F_Zu"}) -> %d", xfd->fd, &sa, salen, ps);*/
@@ -254,8 +257,8 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
 	       ps, pa, pas, strerror(errno));
       }
       Notice2("accepting connection from %s on %s",
-	      sockaddr_info(&pa->soa, peername, sizeof(peername)),
-	      sockaddr_info(&la->soa, sockname, sizeof(sockname)));
+	      sockaddr_info(&pa->soa, pas, peername, sizeof(peername)),
+	      sockaddr_info(&la->soa, las, sockname, sizeof(sockname)));
 
       if (xiocheckpeer(xfd, pa, la) < 0) {
 	 if (Shutdown(ps, 2) < 0) {
@@ -265,7 +268,7 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
       }
 
       Info1("permitting connection from %s",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, pas,
 			  infobuff, sizeof(infobuff)));
 
       applyopts(xfd->fd, opts, PH_FD);
@@ -324,7 +327,6 @@ int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, sockl
    if ((result = _xio_openlate(xfd, opts)) < 0)
       return result;
 
-   xfd->howtoend = END_SHUTDOWN;
    return 0;
 }
 
