@@ -1,5 +1,5 @@
-/* $Id: xio-openssl.c,v 1.28 2006/06/30 20:51:56 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2006 */
+/* $Id: xio-openssl.c,v 1.33 2007/02/26 21:31:40 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the implementation of the openssl addresses */
@@ -183,8 +183,9 @@ static int
    hostname = argv[1];
    portname = argv[2];
 
-   applyopts(-1, opts, PH_INIT);
+   xfd->howtoend = END_SHUTDOWN;
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    retropt_bool(opts, OPT_FORK, &dofork);
 
@@ -392,7 +393,7 @@ static int
    /*! lowport? */
    int level;
    SSL_CTX* ctx;
-   bool opt_ver = false;	/* dont verify peer certificate */
+   bool opt_ver = true;	/* verify peer certificate - changed with 1.6.0 */
    char *opt_cert = NULL;	/* file name of server certificate */
    int result;
 
@@ -417,8 +418,9 @@ static int
    
    portname = argv[1];
 
-   applyopts(-1, opts, PH_INIT);
+   xfd->howtoend = END_SHUTDOWN;
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    retropt_string(opts, OPT_OPENSSL_CERTIFICATE, &opt_cert);
    if (opt_cert == NULL) {
@@ -440,7 +442,7 @@ static int
    }
 
    xfd->addr  = &addr_openssl_listen;
-   xfd->dtype = DATA_OPENSSL;
+   xfd->dtype = XIODATA_OPENSSL;
 
    while (true) {	/* loop over failed attempts */
 
@@ -633,7 +635,7 @@ int
    int result;
 
    xfd->addr  = &addr_openssl;
-   xfd->dtype = DATA_OPENSSL;
+   xfd->dtype = XIODATA_OPENSSL;
 
    retropt_bool(opts, OPT_OPENSSL_FIPS, &opt_fips);
    retropt_string(opts, OPT_OPENSSL_METHOD, &me_str);
@@ -1090,6 +1092,11 @@ ssize_t xioread_openssl(struct single *pipe, void *buff, size_t bufsiz) {
    return ret;
 }
 
+ssize_t xiopending_openssl(struct single *pipe) {
+   int bytes = sycSSL_pending(pipe->para.openssl.ssl);
+   return bytes;
+}
+
 /* on result < 0: errno is set (at least to EIO) */
 ssize_t xiowrite_openssl(struct single *pipe, const void *buff, size_t bufsiz) {
    unsigned long err;
