@@ -1,5 +1,5 @@
-/* $Id: xio-progcall.c,v 1.50 2006/06/30 20:35:11 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-progcall.c,v 1.54 2007/03/06 21:11:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains common code dealing with program calls (exec, system) */
@@ -71,8 +71,8 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
    bool withfork;
 
    popts = moveopts(*copts, GROUP_ALL);
-   applyopts2(-1, popts, PH_INIT, PH_EARLY);
    if (applyopts_single(fd, popts, PH_INIT) < 0)  return -1;
+   applyopts2(-1, popts, PH_INIT, PH_EARLY);
 
    retropt_bool(popts, OPT_NOFORK, &nofork);
    withfork = !nofork;
@@ -201,7 +201,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 #elif HAVE_DEV_PTC
 #  define PTMX "/dev/ptc"	/* AIX 4.3.3 */
 #endif
-      fd->dtype = DATA_PTY;
+      fd->dtype = XIODATA_PTY;
 #if HAVE_DEV_PTMX || HAVE_DEV_PTC
       if (usebestpty || useptmx) {
 	 if ((ptyfd = Open(PTMX, O_RDWR|O_NOCTTY, 0620)) < 0) {
@@ -242,11 +242,16 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	       /*0 Info2("open(\"%s\", O_RDWR|O_NOCTTY, 0620) -> %d", tn, ttyfd);*/
 	    }
 
-#if __unix && __sun
-	    /*! solaris */
-	    Ioctl(ttyfd, I_PUSH, "ptem");
-	    Ioctl(ttyfd, I_PUSH, "ldterm");
-	    Ioctl(ttyfd, I_PUSH, "ttcompat");
+#ifdef I_PUSH
+	    /* Linux: I_PUSH def'd; pty: ioctl(, I_FIND, ...) -> -1 EINVAL */
+	    /* AIX:   I_PUSH def'd; pty: ioctl(, I_FIND, ...) -> 1 */
+	    /* SunOS: I_PUSH def'd; pty: ioctl(, I_FIND, ...) -> 0 */
+	    /* HP-UX: I_PUSH def'd; pty: ioctl(, I_FIND, ...) -> 0 */
+	    if (Ioctl(ttyfd, I_FIND, "ldterm") == 0) {
+	       Ioctl(ttyfd, I_PUSH, "ptem");		/* 0 */
+	       Ioctl(ttyfd, I_PUSH, "ldterm");		/* 0 */
+	       Ioctl(ttyfd, I_PUSH, "ttcompat");	/* HP-UX: -1 */
+	    }
 #endif
 
 #if 0	    /* the following block need not work */
@@ -278,7 +283,9 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	 return STAT_RETRYLATER;
       }
       applyopts_cloexec(ptyfd, popts);/*!*/
-      fd->howtoend = END_CLOSE_KILL;
+      if (fd->howtoend = END_UNSPEC) {
+	 fd->howtoend = END_CLOSE_KILL;
+      }
 
       /* this for parent, was after fork */
       applyopts(ptyfd, popts, PH_FD);
@@ -295,7 +302,7 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       struct opt *popts2, *copts2;
 
       if (rw == XIO_RDWR)
-	 fd->dtype = DATA_2PIPE;
+	 fd->dtype = XIODATA_2PIPE;
       if (rw != XIO_WRONLY) {
 	 if (Pipe(rdpip) < 0) {
 	    Error2("pipe(%p): %s", rdpip, strerror(errno));
@@ -333,7 +340,9 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
 	 applyopts(wrpip[1], popts2, PH_FD);
 	 applyopts(wrpip[0], copts2, PH_FD);
       }
-      fd->howtoend = END_CLOSE_KILL;
+      if (fd->howtoend == END_UNSPEC) {
+	 fd->howtoend = END_CLOSE_KILL;
+      }
 
       /* this for parent, was after fork */
       switch (rw) {
@@ -376,7 +385,9 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       applyopts(sv[1], popts, PH_BIND);
       applyopts(sv[1], popts, PH_PASTBIND);
 
-      fd->howtoend = END_SHUTDOWN_KILL;
+      if (fd->howtoend == END_UNSPEC) {
+	 fd->howtoend = END_SHUTDOWN_KILL;
+      }
 
       /* this for parent, was after fork */
       fd->fd = sv[0];
