@@ -1,5 +1,5 @@
-/* $Id: xio-progcall.h,v 1.12 2003/12/23 21:22:38 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2003 */
+/* $Id: xio-progcall.h,v 1.13 2006/02/08 19:37:15 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_progcall_h_included
@@ -18,8 +18,7 @@ extern const struct optdesc opt_sighup;
 extern const struct optdesc opt_sigint;
 extern const struct optdesc opt_sigquit;
 
-extern int _xioopen_foxec(char *comma,
-		int rw,	/* O_RDONLY etc. */
+extern int _xioopen_foxec(int rw,	/* O_RDONLY etc. */
 		struct single *fd,
 		unsigned groups,
 		struct opt **opts
