@@ -1,4 +1,4 @@
-/* $Id: xio-proxy.c,v 1.26 2006/07/13 06:46:19 gerhard Exp $ */
+/* $Id: xio-proxy.c,v 1.28 2006/12/28 14:02:54 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2002-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -108,8 +108,9 @@ static int xioopen_proxy_connect(int argc, const char *argv[], struct opt *opts,
    targetname = argv[2];
    targetport = argv[3];
 
-   applyopts(-1, opts, PH_INIT);
+   xfd->howtoend = END_SHUTDOWN;
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
@@ -240,6 +241,9 @@ int _xioopen_proxy_prepare(struct proxyvars *proxyvars, struct opt *opts,
    retropt_string(opts, OPT_PROXY_AUTHORIZATION, &proxyvars->authstring);
 
    if (proxyvars->doresolve) {
+      /* currently we only resolve to IPv4 addresses. This is in accordance to
+	 RFC 2396; however once it becomes clear how IPv6 addresses should be
+	 represented in CONNECT commands this code might be extended */
       host = Gethostbyname(targetname);
       if (host == NULL) {
 	 int level = E_WARN;
