@@ -1,5 +1,5 @@
-/* $Id: xio-pty.c,v 1.18 2006/05/31 19:16:02 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2006 */
+/* $Id: xio-pty.c,v 1.20 2007/01/25 21:36:11 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for creating pty addresses */
@@ -47,6 +47,8 @@ static int xioopen_pty(int argc, const char *argv[], struct opt *opts, int xiofl
 				   compatibility we choose "no" as default */
    struct timespec pollintv = { PTY_INTERVALL };
 
+   xfd->stream.howtoend = END_CLOSE;
+
    applyopts(-1, opts, PH_INIT);
    if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
 
@@ -160,8 +162,7 @@ static int xioopen_pty(int argc, const char *argv[], struct opt *opts, int xiofl
    applyopts_named(ptyname, opts, PH_FD);
 
    applyopts_cloexec(ptyfd, opts);/*!*/
-   xfd->stream.dtype    = DATA_PTY;
-   xfd->stream.howtoend = END_CLOSE;
+   xfd->stream.dtype    = XIODATA_PTY;
 
    applyopts(ptyfd, opts, PH_FD);
    
