@@ -1,5 +1,5 @@
-/* $Id: xio-rawip.c,v 1.26 2006/07/13 06:46:55 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-rawip.c,v 1.32 2007/02/05 19:56:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of raw IP type */
@@ -12,10 +12,20 @@
 #include "xio-socket.h"
 #include "xio-ip.h"
 #include "xio-ip6.h"
+#include "xio-tcpwrap.h"
 
 #include "xio-rawip.h"
 
 
+
+static
+int xioopen_rawip_sendto(int argc, const char *argv[], struct opt *opts,
+			 int xioflags, xiofile_t *fd, unsigned groups, int pf,
+			 int dummy2, int dummy3);
+static
+int xioopen_rawip_datagram(int argc, const char *argv[], struct opt *opts,
+			 int xioflags, xiofile_t *fd, unsigned groups, int pf,
+			 int dummy2, int dummy3);
 static
 int xioopen_rawip_recvfrom(int argc, const char *argv[], struct opt *opts,
 			 int xioflags, xiofile_t *xfd, unsigned groups,
@@ -25,18 +35,26 @@ int xioopen_rawip_recv(int argc, const char *argv[], struct opt *opts,
 		       int xioflags, xiofile_t *xfd, unsigned groups,
 		       int pf, int socktype, int ipproto);
 
+static
+int _xioopen_rawip_sendto(const char *hostname, const char *protname,
+			  struct opt *opts, int xioflags,
+			  xiofile_t *xxfd, unsigned groups, int pf);
+
 const struct addrdesc addr_rawip_sendto  = { "ip-sendto",      3, xioopen_rawip_sendto,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6, PF_UNSPEC, 0, 0 HELP(":<host>:<protocol>") };
+const struct addrdesc addr_rawip_datagram= { "ip-datagram",    3, xioopen_rawip_datagram, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_RANGE, PF_UNSPEC, 0, 0 HELP(":<host>:<protocol>") };
 const struct addrdesc addr_rawip_recvfrom= { "ip-recvfrom",    3, xioopen_rawip_recvfrom, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_CHILD|GROUP_RANGE, PF_UNSPEC, SOCK_RAW, 0 HELP(":<protocol>") };
 const struct addrdesc addr_rawip_recv    = { "ip-recv",        1, xioopen_rawip_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_RANGE,             PF_UNSPEC, SOCK_RAW, 0 HELP(":<protocol>") };
 
 #if WITH_IP4
 const struct addrdesc addr_rawip4_sendto  = { "ip4-sendto",     3, xioopen_rawip_sendto,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4, PF_INET, 0, 0 HELP(":<host>:<protocol>") };
+const struct addrdesc addr_rawip4_datagram= { "ip4-datagram",   3, xioopen_rawip_datagram, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_RANGE, PF_INET, 0, 0 HELP(":<host>:<protocol>") };
 const struct addrdesc addr_rawip4_recvfrom= { "ip4-recvfrom",   3, xioopen_rawip_recvfrom, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_CHILD|GROUP_RANGE, PF_INET,  SOCK_RAW, 0 HELP(":<protocol>") };
 const struct addrdesc addr_rawip4_recv    = { "ip4-recv",       1, xioopen_rawip_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_RANGE,             PF_INET,  SOCK_RAW, 0 HELP(":<protocol>") };
 #endif
 
 #if WITH_IP6
 const struct addrdesc addr_rawip6_sendto  = { "ip6-sendto",     3, xioopen_rawip_sendto,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6, PF_INET6, 0, 0 HELP(":<host>:<protocol>") };
+const struct addrdesc addr_rawip6_datagram= { "ip6-datagram",   3, xioopen_rawip_datagram, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_RANGE, PF_INET6, 0, 0 HELP(":<host>:<protocol>") };
 const struct addrdesc addr_rawip6_recvfrom= { "ip6-recvfrom",   3, xioopen_rawip_recvfrom, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_CHILD|GROUP_RANGE, PF_INET6, SOCK_RAW, 0 HELP(":<protocol>") };
 const struct addrdesc addr_rawip6_recv    = { "ip6-recv",       1, xioopen_rawip_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_RANGE,             PF_INET6, SOCK_RAW, 0 HELP(":<protocol>") };
 #endif
@@ -45,41 +63,59 @@ const struct addrdesc addr_rawip6_recv    = { "ip6-recv",       1, xioopen_rawip
 /* we expect the form: host:protocol */
 /* struct sockaddr_in sa;*/
 /* socklen_t salen;*/
-int xioopen_rawip_sendto(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *xxfd, unsigned groups, int pf, int dummy2, int dummy3) {
-   const char *protname = argv[2];
+static
+int xioopen_rawip_sendto(int argc, const char *argv[], struct opt *opts,
+			 int xioflags, xiofile_t *xxfd, unsigned groups,
+			 int pf, int dummy2, int dummy3) {
+   int result;
+
+   if (argc != 3) {
+      Error2("%s: wrong number of parameters (%d instead of 2)",
+	     argv[0], argc-1);
+      return STAT_NORETRY;
+   }
+   if ((result = _xioopen_rawip_sendto(argv[1], argv[2], opts, xioflags, xxfd,
+				       groups, pf)) != STAT_OK) {
+      return result;
+   }
+   _xio_openlate(&xxfd->stream, opts);
+   return STAT_OK;
+}
+
+static
+int _xioopen_rawip_sendto(const char *hostname, const char *protname,
+			  struct opt *opts, int xioflags, xiofile_t *xxfd,
+			  unsigned groups, int pf) {
    char *garbage;
    xiosingle_t *xfd = &xxfd->stream;
    union sockaddr_union us;
    socklen_t uslen;
-   int feats = 1;
+   int feats = 1;	/* option bind supports only address, not port */
    int socktype = SOCK_RAW;
    int ipproto;
    bool needbind = false;
    int result;
 
-   if (argc != 3) {
-      Error2("%s: wrong number of parameters (%d instead of 2)", argv[0], argc-1);
-      return STAT_NORETRY;
-   }
-
    if ((ipproto = strtoul(protname, &garbage, 0)) >= 256) {
       Error3("xioopen_rawip_sendto(\"%s:%s\",,): protocol number exceeds 255 (%u)",
-	     argv[1], protname, ipproto);
+	     hostname, protname, ipproto);
       return STAT_NORETRY;
    } else if (*garbage) {
       Warn2("xioopen_rawip_sendto(\"%s:%s\",,): trailing garbage in protocol specification",
-	     argv[1], protname);
+	     hostname, protname);
       /*return STAT_NORETRY;*/
    }
+
+   xfd->howtoend = END_SHUTDOWN;
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
    /* ...res_opts[] */
-   applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    xfd->salen = sizeof(xfd->peersa);
    if ((result =
-	xiogetaddrinfo(argv[1], NULL, pf, socktype, ipproto,
+	xiogetaddrinfo(hostname, NULL, pf, socktype, ipproto,
 		       &xfd->peersa, &xfd->salen,
 		       xfd->para.socket.ip.res_opts[0],
 		       xfd->para.socket.ip.res_opts[1]))
@@ -91,20 +127,67 @@ int xioopen_rawip_sendto(int argc, const char *argv[], struct opt *opts, int xio
    }
 
    uslen = socket_init(pf, &us);
-   xfd->howtoend = END_SHUTDOWN;
 
-   xfd->dtype = DATA_RECVFROM_SKIPIP;
+   xfd->dtype = XIODATA_RECVFROM_SKIPIP;
 
-   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, feats) >= 0) {
+   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, feats,
+		    xfd->para.socket.ip.res_opts[0],
+		    xfd->para.socket.ip.res_opts[1]) != STAT_NOACTION) {
       needbind = true;
    }
    return
-      _xioopen_dgram_sendto(&xfd->peersa, xfd->salen,
-			  needbind?&us:NULL, uslen,
+      _xioopen_dgram_sendto(needbind?&us:NULL, uslen,
 			  opts, xioflags, xfd, groups, pf, socktype, ipproto);
 }
 
 
+/* we expect the form: address:protocol */
+static
+int xioopen_rawip_datagram(int argc, const char *argv[], struct opt *opts,
+			 int xioflags, xiofile_t *xxfd, unsigned groups,
+			 int pf, int dummy2, int dummy3) {
+   xiosingle_t *xfd = &xxfd->stream;
+   char *rangename;
+   int result;
+
+   if (argc != 3) {
+      Error2("%s: wrong number of parameters (%d instead of 2)",
+	     argv[0], argc-1);
+      return STAT_NORETRY;
+   }
+   if ((result =
+	_xioopen_rawip_sendto(argv[1], argv[2], opts, xioflags, xxfd,
+				groups, pf)) != STAT_OK) {
+      return result;
+   }
+
+   xfd->dtype = XIOREAD_RECV|XIOWRITE_SENDTO;
+   if (pf == PF_INET) {
+      xfd->dtype |= XIOREAD_RECV_SKIPIP;
+   }
+
+   xfd->para.socket.la.soa.sa_family = xfd->peersa.soa.sa_family;
+
+   /* which reply packets will be accepted - determine by range option */
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, pf, &xfd->para.socket.range) < 0) {
+	 free(rangename);
+	 return STAT_NORETRY;
+      }
+      xfd->para.socket.dorange = true;
+      xfd->dtype |= XIOREAD_RECV_CHECKRANGE;
+      free(rangename);
+   }
+
+#if WITH_LIBWRAP
+   xio_retropt_tcpwrap(xfd, opts);
+#endif /* WITH_LIBWRAP */
+
+   _xio_openlate(xfd, opts);
+   return STAT_OK;
+}
+
+
 static
 int xioopen_rawip_recvfrom(int argc, const char *argv[], struct opt *opts,
 		     int xioflags, xiofile_t *xfd, unsigned groups,
@@ -115,6 +198,7 @@ int xioopen_rawip_recvfrom(int argc, const char *argv[], struct opt *opts,
    socklen_t uslen = sizeof(us);
    int ipproto;
    bool needbind = false;
+   int result;
 
    if (argc != 2) {
       Error2("%s: wrong number of parameters (%d instead of 1)",
@@ -131,6 +215,7 @@ int xioopen_rawip_recvfrom(int argc, const char *argv[], struct opt *opts,
 	     protname);
       /*return STAT_NORETRY;*/
    }
+   xfd->stream.howtoend = END_NONE;
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
    retropt_socket_pf(opts, &pf);
@@ -144,14 +229,21 @@ int xioopen_rawip_recvfrom(int argc, const char *argv[], struct opt *opts,
 #endif
    }
 
-   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, 1) >= 0) {
+   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, 1,
+		    xfd->stream.para.socket.ip.res_opts[0],
+		    xfd->stream.para.socket.ip.res_opts[1]) != STAT_NOACTION) {
       needbind = true;
    }
 
-   xfd->stream.dtype = DATA_RECVFROM_SKIPIP_ONE;
-   return _xioopen_dgram_recvfrom(&xfd->stream, xioflags,
-				needbind?&us.soa:NULL, uslen,
-			      opts, pf, socktype, ipproto, E_ERROR);
+   xfd->stream.dtype = XIODATA_RECVFROM_SKIPIP_ONE;
+   if ((result =
+	_xioopen_dgram_recvfrom(&xfd->stream, xioflags, needbind?&us.soa:NULL,
+				uslen, opts, pf, socktype, ipproto, E_ERROR))
+       != STAT_OK) {
+      return result;
+   }
+   _xio_openlate(&xfd->stream, opts);
+   return STAT_OK;
 }
 
 
@@ -193,15 +285,18 @@ int xioopen_rawip_recv(int argc, const char *argv[], struct opt *opts,
 #endif
    }
 
-   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, 1) !=
+   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, 1,
+		    xfd->stream.para.socket.ip.res_opts[0],
+		    xfd->stream.para.socket.ip.res_opts[1]) !=
        STAT_OK) {
       /* pf is required during xioread checks */
       xfd->stream.para.socket.la.soa.sa_family = pf;
    }
 
-   xfd->stream.dtype = DATA_RECV_SKIPIP;
+   xfd->stream.dtype = XIODATA_RECV_SKIPIP;
    result = _xioopen_dgram_recv(&xfd->stream, xioflags, NULL/*&us.soa*/, uslen,
 				opts, pf, socktype, ipproto, E_ERROR);
+   _xio_openlate(&xfd->stream, opts);
    return result;
 }
 
