@@ -1,20 +1,21 @@
-/* $Id: xio-rawip.h,v 1.10 2006/05/06 14:19:53 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-rawip.h,v 1.13 2007/02/05 19:56:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_rawip_h_included
 #define __xio_rawip_h_included 1
 
 extern const struct addrdesc addr_rawip_sendto;
+extern const struct addrdesc addr_rawip_datagram;
 extern const struct addrdesc addr_rawip_recvfrom;
 extern const struct addrdesc addr_rawip_recv;
 extern const struct addrdesc addr_rawip4_sendto;
+extern const struct addrdesc addr_rawip4_datagram;
 extern const struct addrdesc addr_rawip4_recvfrom;
 extern const struct addrdesc addr_rawip4_recv;
 extern const struct addrdesc addr_rawip6_sendto;
+extern const struct addrdesc addr_rawip6_datagram;
 extern const struct addrdesc addr_rawip6_recvfrom;
 extern const struct addrdesc addr_rawip6_recv;
 
-extern int xioopen_rawip_sendto(int argc, const char *argv[], struct opt *opts, int xioflags, xiofile_t *fd, unsigned groups, int pf, int dummy2, int dummy3);
-
 #endif /* !defined(__xio_rawip_h_included) */
