@@ -1,5 +1,5 @@
-/* $Id: xio-readline.c,v 1.15 2006/03/21 19:15:13 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2006 */
+/* $Id: xio-readline.c,v 1.17 2007/01/25 21:36:11 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening the readline address */
@@ -71,7 +71,7 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
 
    xfd->stream.fd = 0;	/* stdin */
    xfd->stream.howtoend = END_NONE;
-   xfd->stream.dtype    = DATA_READLINE;
+   xfd->stream.dtype    = XIODATA_READLINE;
 
 #if WITH_TERMIOS
    if (Isatty(xfd->stream.fd)) {
@@ -84,8 +84,8 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    }
 #endif /* WITH_TERMIOS */
 
-   applyopts(-1, opts, PH_INIT);
    if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    applyopts2(xfd->stream.fd, opts, PH_INIT, PH_FD);
 
@@ -203,7 +203,7 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
 }
 
 void xioscan_readline(struct single *pipe, const void *buff, size_t bytes) {
-   if (pipe->dtype == DATA_READLINE && pipe->para.readline.dynprompt) {
+   if (pipe->dtype == XIODATA_READLINE && pipe->para.readline.dynprompt) {
       /* we save the last part of the output as possible prompt */
       const void *ptr = buff;
       const void *pcr = memrchr(buff, '\r', bytes);
