@@ -1,12 +1,12 @@
-/* $Id: xio-socket.c,v 1.39 2006/07/13 06:47:44 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-socket.c,v 1.45 2007/03/06 21:12:09 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for socket related functions */
 
 #include "xiosysincludes.h"
 
-#if WITH_SOCKET
+#if _WITH_SOCKET
 
 #include "xioopen.h"
 #include "xio-socket.h"
@@ -217,7 +217,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	 if (Bind(xfd->fd, (struct sockaddr *)sinp, sizeof(*sinp)) < 0) {
 	    Msg4(errno==EADDRINUSE?E_INFO:level,
 		 "bind(%d, {%s}, "F_Zd"): %s", xfd->fd,
-		 sockaddr_info(&sinp->soa, infobuff, sizeof(infobuff)),
+		 sockaddr_info(&sinp->soa, sizeof(*sinp), infobuff, sizeof(infobuff)),
 		 sizeof(*sinp), strerror(errno));
 	    if (errno != EADDRINUSE) {
 	       Close(xfd->fd);
@@ -240,7 +240,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
    if (us) {
       if (Bind(xfd->fd, us, uslen) < 0) {
 	 Msg4(level, "bind(%d, {%s}, "F_Zd"): %s",
-	      xfd->fd, sockaddr_info(us, infobuff, sizeof(infobuff)),
+	      xfd->fd, sockaddr_info(us, uslen, infobuff, sizeof(infobuff)),
 	      uslen, strerror(errno));
 	 Close(xfd->fd);
 	 return STAT_RETRYLATER;
@@ -259,9 +259,9 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 
    result = Connect(xfd->fd, (struct sockaddr *)them, themlen);
    _errno = errno;
-   la.soa.sa_family = them->sa_family;
+   la.soa.sa_family = them->sa_family;  lalen = sizeof(la);
    if (Getsockname(xfd->fd, &la.soa, &lalen) < 0) {
-      Warn4("getsockname(%d, %p, {%d}): %s",
+      Msg4(level-1, "getsockname(%d, %p, {%d}): %s",
 	    xfd->fd, &la.soa, lalen, strerror(errno));
    }
    errno = _errno;
@@ -273,7 +273,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	    fd_set readfds, writefds, exceptfds;
 	    int result;
 	    Info4("connect(%d, %s, "F_Zd"): %s",
-		  xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		  xfd->fd, sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 		  themlen, strerror(errno));
 	    timeout = xfd->para.socket.connect_timeout;
 	    FD_ZERO(&readfds);  FD_ZERO(&writefds);  FD_ZERO(&exceptfds);
@@ -286,7 +286,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	    }
 	    if (result == 0) {
 	       Msg2(level, "connecting to %s: %s",
-		    sockaddr_info(them, infobuff, sizeof(infobuff)),
+		    sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 		    strerror(ETIMEDOUT));
 	       return STAT_RETRYLATER;
 	    }
@@ -300,7 +300,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 #else
 	       Connect(xfd->fd, them, themlen);	/* get error message */
 	       Msg4(level, "connect(%d, %s, "F_Zd"): %s",
-		     xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		     xfd->fd, sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 		     themlen, strerror(errno));
 #endif
 	       return STAT_RETRYLATER;
@@ -309,7 +309,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	    Fcntl_l(xfd->fd, F_SETFL, fcntl_flags);
 	 } else {
 	    Warn4("connect(%d, %s, "F_Zd"): %s",
-		  xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		  xfd->fd, sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 		  themlen, strerror(errno));
 	 }
       } else if (pf == PF_UNIX && errno == EPROTOTYPE) {
@@ -317,7 +317,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	    the only way to distinguish stream and datagram sockets */
 	 int _errno = errno;
 	 Info4("connect(%d, %s, "F_Zd"): %s",
-	       xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+	       xfd->fd, sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 	       themlen, strerror(errno));
 #if 0
 	 Info("assuming datagram socket");
@@ -331,22 +331,19 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	 return -1;
       } else {
 	 Msg4(level, "connect(%d, %s, "F_Zd"): %s",
-	      xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+	      xfd->fd, sockaddr_info(them, themlen, infobuff, sizeof(infobuff)),
 	      themlen, strerror(errno));
 	 Close(xfd->fd);
 	 return STAT_RETRYLATER;
       }
    }
 
-   xfd->howtoend = END_SHUTDOWN;
-
    applyopts_fchown(xfd->fd, opts);
    applyopts(xfd->fd, opts, PH_CONNECTED);
    applyopts(xfd->fd, opts, PH_LATE);
 
-   xfd->howtoend = END_SHUTDOWN;
    Notice1("successfully connected from local address %s",
-	   sockaddr_info(&la.soa, infobuff, sizeof(infobuff)));
+	   sockaddr_info(&la.soa, themlen, infobuff, sizeof(infobuff)));
 
    return STAT_OK;
 }
@@ -373,7 +370,7 @@ int xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
    opts0 = copyopts(opts, GROUP_ALL);
 
    Notice1("opening connection to %s",
-	   sockaddr_info(them, infobuff, sizeof(infobuff)));
+	   sockaddr_info(them, themlen, infobuff, sizeof(infobuff)));
 
    do {	/* loop over retries and forks */
 
@@ -452,7 +449,7 @@ int xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 
 
 /* common to xioopen_udp_sendto, ..unix_sendto, ..rawip */
-int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
+int _xioopen_dgram_sendto(/* them is already in xfd->peersa */
 			union sockaddr_union *us, socklen_t uslen,
 			struct opt *opts,
 			int xioflags, xiosingle_t *xfd, unsigned groups,
@@ -468,6 +465,7 @@ int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
    }
 
    applyopts_offset(xfd, opts);
+   applyopts_single(xfd, opts, PH_PASTSOCKET);
    applyopts(xfd->fd, opts, PH_PASTSOCKET);
    applyopts(xfd->fd, opts, PH_FD);
 
@@ -479,7 +477,7 @@ int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
    if (us) {
       if (Bind(xfd->fd, (struct sockaddr *)us, uslen) < 0) {
 	 Msg4(level, "bind(%d, {%s}, "F_Zd"): %s",
-	      xfd->fd, sockaddr_info((struct sockaddr *)us, infobuff, sizeof(infobuff)),
+	      xfd->fd, sockaddr_info((struct sockaddr *)us, uslen, infobuff, sizeof(infobuff)),
 	      uslen, strerror(errno));
 	 Close(xfd->fd);
 	 return STAT_RETRYLATER;
@@ -495,18 +493,13 @@ int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
 	    xfd->fd, &la.soa, lalen, strerror(errno));
    }
 
-   xfd->howtoend = END_SHUTDOWN;
-
    applyopts_fchown(xfd->fd, opts);
    applyopts(xfd->fd, opts, PH_CONNECTED);
    applyopts(xfd->fd, opts, PH_LATE);
 
-   xfd->howtoend = END_SHUTDOWN;
    /* xfd->dtype = DATA_RECVFROM; *//* no, the caller must set this (ev _SKIPIP) */
-   xfd->peersa = *them;
-   xfd->salen = themlen;
    Notice1("successfully prepared local socket %s",
-	   sockaddr_info(&la.soa, infobuff, sizeof(infobuff)));
+	   sockaddr_info(&la.soa, lalen, infobuff, sizeof(infobuff)));
 
    return STAT_OK;
 }
@@ -515,7 +508,9 @@ int _xioopen_dgram_sendto(union sockaddr_union *them, socklen_t themlen,
 static pid_t xio_waitingfor;
 static bool xio_hashappened;
 void xiosigaction_hasread(int signum, siginfo_t *siginfo, void *ucontext) {
-   Debug("xiosigaction_hasread()");
+   Debug5("xiosigaction_hasread(%d, {%d,%d,%d,"F_pid"}, )",
+	  signum, siginfo->si_signo, siginfo->si_errno, siginfo->si_code,
+	  siginfo->si_pid);
    if (xio_waitingfor == siginfo->si_pid) {
       xio_hashappened = true;
    }
@@ -526,7 +521,7 @@ void xiosigaction_hasread(int signum, siginfo_t *siginfo, void *ucontext) {
 
 /* waits for incoming packet, checks its source address and port. Depending
    on fork option, it may fork a subprocess.
-   Returns 0 if a the packet was accepted; with fork option, this is already in
+   Returns STAT_OK if a the packet was accepted; with fork option, this is already in
    a new subprocess!
    Other return values indicate a problem; this can happen in the master
    process or in a subprocess.
@@ -538,6 +533,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 			  struct sockaddr *us, socklen_t uslen,
 			  struct opt *opts,
 			  int pf, int socktype, int proto, int level) {
+   char *rangename;
    socklen_t salen;
    bool dofork = false;
    pid_t pid;	/* mostly int; only used with fork */
@@ -556,7 +552,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
       xfd->flags |= XIO_DOESFORK;
    }
 
-   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return STAT_NORETRY;
 
 #if 1
    if (dofork) {
@@ -587,6 +583,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
       return STAT_RETRYLATER;
    }
 
+   applyopts_single(xfd, opts, PH_PASTSOCKET);
    applyopts(xfd->fd, opts, PH_PASTSOCKET);
 
    applyopts_cloexec(xfd->fd, opts);
@@ -595,7 +592,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
    applyopts(xfd->fd, opts, PH_BIND);
    if ((us != NULL) && Bind(xfd->fd, (struct sockaddr *)us, uslen) < 0) {
       Msg4(level, "bind(%d, {%s}, "F_Zd"): %s", xfd->fd,
-	   sockaddr_info(us, infobuff, sizeof(infobuff)), uslen,
+	   sockaddr_info(us, uslen, infobuff, sizeof(infobuff)), uslen,
 	   strerror(errno));
       Close(xfd->fd);
       return STAT_RETRYLATER;
@@ -617,11 +614,14 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 #endif /* WITH_UNIX */
 
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &xfd->para.socket.rangename) >= 0) {
-      if (parserange(xfd->para.socket.rangename, pf, &xfd->para.socket.range)
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, pf, &xfd->para.socket.range)
 	  < 0) {
+	 free(rangename);
 	 return STAT_NORETRY;
       }
+      free(rangename);
+      xfd->para.socket.dorange = true;
    }
 #endif
 
@@ -680,7 +680,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
       union sockaddr_union _sockname;
       union sockaddr_union *pa = &_peername;	/* peer address */
       union sockaddr_union *la = &_sockname;	/* local address */
-      socklen_t pas = sizeof(_peername);	/* peer address size */
+      socklen_t palen = sizeof(_peername);	/* peer address size */
 
       socket_init(pf, pa);
       salen = sizeof(struct sockaddr);
@@ -697,7 +697,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 	 fd_set in, out, expt;
 	 /*? int level = E_ERROR;*/
 	 if (us != NULL) {
-	    Notice1("receiving on %s", sockaddr_info(us, lisname, sizeof(lisname)));
+	    Notice1("receiving on %s", sockaddr_info(us, uslen, lisname, sizeof(lisname)));
 	 } else {
 	    Notice1("receiving IP protocol %u", proto);
 	 }
@@ -716,12 +716,12 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 	 return STAT_RETRYLATER;
       } while (true);
 
-      if (xiogetpacketsrc(xfd->fd, pa, &pas) < 0) {
+      if (xiogetpacketsrc(xfd->fd, pa, &palen) < 0) {
 	 return STAT_RETRYLATER;
       }
 
       Notice1("receiving packet from %s"/*"src"*/,
-	      sockaddr_info((struct sockaddr *)pa, peername, sizeof(peername))/*,
+	      sockaddr_info((struct sockaddr *)pa, palen, peername, sizeof(peername))/*,
 							     sockaddr_info(&la->soa, sockname, sizeof(sockname))*/);
 
       if (xiocheckpeer(xfd, pa, la) < 0) {
@@ -731,7 +731,7 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 	 continue;
       }
       Info1("permitting packet from %s",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, palen,
 			  infobuff, sizeof(infobuff)));
 
       applyopts(xfd->fd, opts, PH_FD);
@@ -739,15 +739,27 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
       applyopts(xfd->fd, opts, PH_CONNECTED);
 
       xfd->peersa = *(union sockaddr_union *)pa;
-      xfd->salen = pas;
+      xfd->salen = palen;
 
       if (dofork) {
+	 sigset_t mask_sigchldusr1;
 	 const char *forkwaitstring;
 	 int forkwaitsecs = 0;
 
+	 /* we must prevent that the current packet triggers another fork;
+	    therefore we wait for a signal from the recent child: USR1
+	    indicates that is has consumed the last packet; CHLD means it has
+	    terminated */
+	 /* block SIGCHLD and SIGUSR1 until parent is ready to react */
+	 sigemptyset(&mask_sigchldusr1);
+	 sigaddset(&mask_sigchldusr1, SIGCHLD);
+	 sigaddset(&mask_sigchldusr1, SIGUSR1);
+	 Sigprocmask(SIG_BLOCK, &mask_sigchldusr1, NULL);
+
 	 if ((pid = Fork()) < 0) {
 	    Msg1(level, "fork(): %s", strerror(errno));
 	    Close(xfd->fd);
+	    Sigprocmask(SIG_UNBLOCK, &mask_sigchldusr1, NULL);
 	    return STAT_RETRYLATER;
 	 }
 	 /* gdb recommends to have env controlled sleep after fork */
@@ -757,6 +769,8 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 	 }
 
 	 if (pid == 0) {	/* child */
+	    /* no reason to block SIGCHLD in child process */
+	    Sigprocmask(SIG_UNBLOCK, &mask_sigchldusr1, NULL);
 	    xfd->ppid = Getppid();	/* send parent a signal when packet has
 					   been consumed */
 
@@ -783,13 +797,15 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 
 	 /* server: continue loop with listen */
 	 Notice1("forked off child process "F_pid, pid);
-	 /* we must prevent that the current packet triggers another fork;
-	    therefore we wait for what ??? */
+
 	 xio_waitingfor = pid;
-	 
-	 do {
-	    Sleep(1);
-	 } while (!xio_hashappened);
+	 /* now we are ready to handle signals */
+	 Sigprocmask(SIG_UNBLOCK, &mask_sigchldusr1, NULL);
+
+	 while (!xio_hashappened) {
+	    Sleep(UINT_MAX);	/* any signal lets us continue */
+	 }
+	 xio_waitingfor = 0;	/* so this child will not set hashappened again */
 	 xio_hashappened = false;
 
 	 Info("continue listening");
@@ -797,21 +813,22 @@ int _xioopen_dgram_recvfrom(struct single *xfd, int xioflags,
 	break;
       }
    }
-   if ((result = _xio_openlate(xfd, opts)) < 0)
-      return result;
+   if ((result = _xio_openlate(xfd, opts)) != 0)
+      return STAT_NORETRY;
 
-   xfd->howtoend = END_NONE;
-   return 0;
+   return STAT_OK;
 }
 
 
+/* returns STAT_* */
 int _xioopen_dgram_recv(struct single *xfd, int xioflags,
 			struct sockaddr *us, socklen_t uslen,
 			struct opt *opts, int pf, int socktype, int proto,
 			int level) {
+   char *rangename;
    char infobuff[256];
 
-   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return STAT_NORETRY;
 
    if ((xfd->fd = Socket(pf, socktype, proto)) < 0) {
       Msg4(level,
@@ -819,6 +836,7 @@ int _xioopen_dgram_recv(struct single *xfd, int xioflags,
       return STAT_RETRYLATER;
    }
 
+   applyopts_single(xfd, opts, PH_PASTSOCKET);
    applyopts(xfd->fd, opts, PH_PASTSOCKET);
 
    applyopts_cloexec(xfd->fd, opts);
@@ -827,7 +845,7 @@ int _xioopen_dgram_recv(struct single *xfd, int xioflags,
    applyopts(xfd->fd, opts, PH_BIND);
    if ((us != NULL) && Bind(xfd->fd, (struct sockaddr *)us, uslen) < 0) {
       Msg4(level, "bind(%d, {%s}, "F_Zd"): %s", xfd->fd,
-	   sockaddr_info(us, infobuff, sizeof(infobuff)), uslen,
+	   sockaddr_info(us, uslen, infobuff, sizeof(infobuff)), uslen,
 	   strerror(errno));
       Close(xfd->fd);
       return STAT_RETRYLATER;
@@ -849,11 +867,14 @@ int _xioopen_dgram_recv(struct single *xfd, int xioflags,
 #endif /* WITH_UNIX */
 
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &xfd->para.socket.rangename) >= 0) {
-      if (parserange(xfd->para.socket.rangename, pf, &xfd->para.socket.range)
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, pf, &xfd->para.socket.range)
 	  < 0) {
+	 free(rangename);
 	 return STAT_NORETRY;
       }
+      free(rangename);
+      xfd->para.socket.dorange = true;
    }
 #endif
 
@@ -868,7 +889,7 @@ int _xioopen_dgram_recv(struct single *xfd, int xioflags,
       Info("starting recvfrom loop");
    }
 
-   return 0;
+   return STAT_OK;
 }
 
 
@@ -956,7 +977,7 @@ int xiogetpacketsrc(int fd, union sockaddr_union *pa, socklen_t *palen) {
 		,
 	       &pa->soa, palen) < 0) {
       Notice1("packet from %s",
-	      sockaddr_info(&pa->soa, infobuff, sizeof(infobuff)));
+	      sockaddr_info(&pa->soa, *palen, infobuff, sizeof(infobuff)));
       Warn1("recvfrom(): %s", strerror(errno));
       return STAT_RETRYLATER;
    }
@@ -988,16 +1009,16 @@ int xiocheckpeer(xiosingle_t *xfd,
    int result;
 
 #if WITH_IP4
-   if (xfd->para.socket.rangename != NULL) {
+   if (xfd->para.socket.dorange) {
       if (xiocheckrange(pa, &xfd->para.socket.range) < 0) {
 	 char infobuff[256];
 	 Warn1("refusing connection from %s due to range option",
-	       sockaddr_info((struct sockaddr *)pa,
+	       sockaddr_info((struct sockaddr *)pa, 0,
 			     infobuff, sizeof(infobuff)));
 	 return -1;
       }
       Info1("permitting connection from %s due to range option",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, 0,
 			  infobuff, sizeof(infobuff)));
    }
 #endif /* WITH_IP4 */
@@ -1007,8 +1028,8 @@ int xiocheckpeer(xiosingle_t *xfd,
 #if WITH_IP4
       if (pa->soa.sa_family == AF_INET &&
 	  ntohs(((struct sockaddr_in *)pa)->sin_port) != xfd->para.socket.ip.sourceport) {
-	 Warn1("refusing connection from %s due to sourceport option",
-	       sockaddr_info((struct sockaddr *)pa,
+	 Warn1("refusing connection from %s due to wrong sourceport",
+	       sockaddr_info((struct sockaddr *)pa, 0,
 			     infobuff, sizeof(infobuff)));
 	 return -1;
       }
@@ -1017,19 +1038,19 @@ int xiocheckpeer(xiosingle_t *xfd,
       if (pa->soa.sa_family == AF_INET6 &&
 	  ntohs(((struct sockaddr_in6 *)pa)->sin6_port) != xfd->para.socket.ip.sourceport) {
 	 Warn1("refusing connection from %s due to sourceport option",
-	       sockaddr_info((struct sockaddr *)pa,
+	       sockaddr_info((struct sockaddr *)pa, 0,
 			     infobuff, sizeof(infobuff)));
 	 return -1;
       }
 #endif /* WITH_IP6 */
       Info1("permitting connection from %s due to sourceport option",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, 0,
 			  infobuff, sizeof(infobuff)));
    } else if (xfd->para.socket.ip.lowport) {
       if (pa->soa.sa_family == AF_INET &&
 	  ntohs(((struct sockaddr_in *)pa)->sin_port) >= IPPORT_RESERVED) {
 	 Warn1("refusing connection from %s due to lowport option",
-	       sockaddr_info((struct sockaddr *)pa,
+	       sockaddr_info((struct sockaddr *)pa, 0,
 			     infobuff, sizeof(infobuff)));
 	 return -1;
       }
@@ -1038,13 +1059,13 @@ int xiocheckpeer(xiosingle_t *xfd,
 	       ntohs(((struct sockaddr_in6 *)pa)->sin6_port) >=
 	       IPPORT_RESERVED) {
 	 Warn1("refusing connection from %s due to lowport option",
-	       sockaddr_info((struct sockaddr *)pa,
+	       sockaddr_info((struct sockaddr *)pa, 0,
 			     infobuff, sizeof(infobuff)));
 	 return -1;
       }
 #endif /* WITH_IP6 */
       Info1("permitting connection from %s due to lowport option",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, 0,
 			  infobuff, sizeof(infobuff)));
    }
 #endif /* WITH_TCP || WITH_UDP */
@@ -1054,12 +1075,12 @@ int xiocheckpeer(xiosingle_t *xfd,
    if (result < 0) {
       char infobuff[256];
       Warn1("refusing connection from %s due to tcpwrapper option",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, 0,
 			  infobuff, sizeof(infobuff)));
       return -1;
    } else if (result > 0) {
       Info1("permitting connection from %s due to tcpwrapper option",
-	    sockaddr_info((struct sockaddr *)pa,
+	    sockaddr_info((struct sockaddr *)pa, 0,
 			  infobuff, sizeof(infobuff)));
    }
 #endif /* (WITH_TCP || WITH_UDP) && WITH_LIBWRAP */
@@ -1067,4 +1088,4 @@ int xiocheckpeer(xiosingle_t *xfd,
    return 0;	/* permitted */
 }
 
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
