@@ -1,5 +1,5 @@
-/* $Id: xio-stdio.h,v 1.4 2001/11/04 17:19:20 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001 */
+/* $Id: xio-stdio.h,v 1.5 2006/02/08 19:47:03 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_stdio_h_included
@@ -7,7 +7,7 @@
 
 
 
-extern int xioopen_stdio_bi(char *arg, xiofile_t *sock);
+extern int xioopen_stdio_bi(xiofile_t *sock);
 
 extern const struct addrdesc addr_stdio;
 extern const struct addrdesc addr_stdin;
