@@ -1,5 +1,5 @@
-/* $Id: xio-udp.c,v 1.29 2006/07/13 06:50:47 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xio-udp.c,v 1.36 2007/02/08 18:27:00 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for handling UDP addresses */
@@ -24,6 +24,10 @@ int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
 		     int xioflags, xiofile_t *xfd, unsigned groups,
 		     int pf, int socktype, int ipproto);
 static
+int xioopen_udp_datagram(int argc, const char *argv[], struct opt *opts,
+		     int xioflags, xiofile_t *xfd, unsigned groups,
+		     int pf, int socktype, int ipproto);
+static
 int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
 		       int xioflags, xiofile_t *xfd, unsigned groups,
 		       int pf, int socktype, int ipproto);
@@ -32,6 +36,12 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
 		     int xioflags, xiofile_t *xfd, unsigned groups,
 		     int pf, int socktype, int ipproto);
 
+static
+int _xioopen_udp_sendto(const char *hostname, const char *servname,
+			struct opt *opts,
+			int xioflags, xiofile_t *xxfd, unsigned groups,
+			int pf, int socktype, int ipproto);
+
 const struct addrdesc addr_udp_connect  = { "udp-connect",    3, xioopen_ipapp_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP, SOCK_DGRAM, IPPROTO_UDP, PF_UNSPEC HELP(":<host>:<port>") };
 #if WITH_LISTEN
 const struct addrdesc addr_udp_listen   = { "udp-listen", 3, xioopen_ipdgram_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE, PF_UNSPEC, IPPROTO_UDP, PF_UNSPEC HELP(":<port>") };
@@ -39,6 +49,7 @@ const struct addrdesc addr_udp_listen   = { "udp-listen", 3, xioopen_ipdgram_lis
 const struct addrdesc addr_udp_sendto   = { "udp-sendto",   3, xioopen_udp_sendto,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP, PF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP HELP(":<host>:<port>") };
 const struct addrdesc addr_udp_recvfrom = { "udp-recvfrom",   3, xioopen_udp_recvfrom,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_CHILD|GROUP_RANGE, PF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP HELP(":<port>") };
 const struct addrdesc addr_udp_recv     = { "udp-recv",       1, xioopen_udp_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_RANGE,             PF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP  HELP(":<port>") };
+const struct addrdesc addr_udp_datagram = { "udp-datagram", 3, xioopen_udp_datagram,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_RANGE, PF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP HELP(":<host>:<port>") };
 
 #if WITH_IP4
 const struct addrdesc addr_udp4_connect = { "udp4-connect",    3, xioopen_ipapp_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP, SOCK_DGRAM, IPPROTO_UDP, PF_INET HELP(":<host>:<port>") };
@@ -46,6 +57,7 @@ const struct addrdesc addr_udp4_connect = { "udp4-connect",    3, xioopen_ipapp_
 const struct addrdesc addr_udp4_listen  = { "udp4-listen", 3, xioopen_ipdgram_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE, PF_INET, IPPROTO_UDP, PF_INET HELP(":<port>") };
 #endif /* WITH_LISTEN */
 const struct addrdesc addr_udp4_sendto  = { "udp4-sendto",     3, xioopen_udp_sendto,   GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP, PF_INET, SOCK_DGRAM, IPPROTO_UDP  HELP(":<host>:<port>") };
+const struct addrdesc addr_udp4_datagram = { "udp4-datagram",3, xioopen_udp_datagram,  GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP|GROUP_RANGE, PF_INET, SOCK_DGRAM, IPPROTO_UDP HELP(":<remote-address>:<port>") };
 const struct addrdesc addr_udp4_recvfrom= { "udp4-recvfrom",   3, xioopen_udp_recvfrom, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP|GROUP_CHILD|GROUP_RANGE, PF_INET, SOCK_DGRAM, IPPROTO_UDP  HELP(":<host>:<port>") };
 const struct addrdesc addr_udp4_recv    = { "udp4-recv",       1, xioopen_udp_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_UDP|GROUP_RANGE,             PF_INET, SOCK_DGRAM, IPPROTO_UDP  HELP(":<port>") };
 #endif /* WITH_IP4 */
@@ -56,6 +68,7 @@ const struct addrdesc addr_udp6_connect = { "udp6-connect",    3, xioopen_ipapp_
 const struct addrdesc addr_udp6_listen  = { "udp6-listen", 3, xioopen_ipdgram_listen, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_LISTEN|GROUP_CHILD|GROUP_RANGE, PF_INET6, IPPROTO_UDP, 0 HELP(":<port>") };
 #endif /* WITH_LISTEN */
 const struct addrdesc addr_udp6_sendto  = { "udp6-sendto",     3, xioopen_udp_sendto, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_UDP, PF_INET6, SOCK_DGRAM, IPPROTO_UDP HELP(":<host>:<port>") };
+const struct addrdesc addr_udp6_datagram= { "udp6-datagram",   3, xioopen_udp_datagram,GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_RANGE, PF_INET6, SOCK_DGRAM, IPPROTO_UDP HELP(":<host>:<port>") };
 const struct addrdesc addr_udp6_recvfrom= { "udp6-recvfrom",   3, xioopen_udp_recvfrom, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_CHILD|GROUP_RANGE, PF_INET6, SOCK_DGRAM, IPPROTO_UDP  HELP(":<port>") };
 const struct addrdesc addr_udp6_recv    = { "udp6-recv",       1, xioopen_udp_recv,     GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP6|GROUP_IP_UDP|GROUP_RANGE,             PF_INET6, SOCK_DGRAM, IPPROTO_UDP  HELP(":<port>") };
 #endif /* WITH_IP6 */
@@ -70,12 +83,11 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    union sockaddr_union us;
    union sockaddr_union themunion;
    union sockaddr_union *them = &themunion;
-   struct hostent *usaddr;
-   char *bindname;
    int socktype = SOCK_DGRAM;
    fd_set in, out, expt;
    bool dofork = false;
    pid_t pid;
+   char *rangename;
    char infobuff[256];
    unsigned char buff1[1];
    socklen_t uslen;
@@ -98,10 +110,16 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 
    retropt_socket_pf(opts, &pf);
 
-   applyopts(-1, opts, PH_INIT);
    if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
    uslen = socket_init(pf, &us);
+   retropt_int(opts, OPT_SO_TYPE, &socktype);
+   retropt_bind(opts, pf, socktype, IPPROTO_UDP,
+		(struct sockaddr *)&us, &uslen, 1,
+		fd->stream.para.socket.ip.res_opts[1],
+		fd->stream.para.socket.ip.res_opts[0]);
+
    if (false) {
       ;
 #if WITH_IP4
@@ -125,51 +143,27 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       }
    }
 
-   if (retropt_string(opts, OPT_BIND, &bindname) >= 0) {
-      usaddr = Gethostbyname(bindname);
-      if (usaddr == NULL) {
-	 Error2("gethostbyname(\"%s\"): %s", bindname,
-		h_errno == NETDB_INTERNAL ? strerror(errno) :
-		hstrerror(h_errno));
-	 return STAT_RETRYLATER;
-      }
-      /*! here we should see what address type we got from gethostbyname() */
-      switch (pf) {
-#if WITH_IP4
-      case PF_INET:  memcpy(&us.ip4.sin_addr, usaddr->h_addr_list[0],  4);
-	 break;
-#endif
-#if WITH_IP6
-      case PF_INET6: memcpy(&us.ip6.sin6_addr, usaddr->h_addr_list[0], 16);
-	 break;
-#endif
-      }
-   }
-
-   retropt_int(opts, OPT_SO_TYPE, &socktype);
-
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &fd->stream.para.socket.rangename) >= 0) {
-      if (parserange(fd->stream.para.socket.rangename, pf,
-		     &fd->stream.para.socket.range) < 0) {
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, pf, &fd->stream.para.socket.range) < 0) {
+	 free(rangename);
 	 return STAT_NORETRY;
       }
+      free(rangename);
+      fd->stream.para.socket.dorange = true;
    }
 #endif
 
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
+#if WITH_LIBWRAP
    xio_retropt_tcpwrap(&fd->stream, opts);
-#endif /* && (WITH_TCP || WITH_UDP) && WITH_LIBWRAP */
+#endif /* WITH_LIBWRAP */
 
-#if WITH_TCP || WITH_UDP
    if (retropt_ushort(opts, OPT_SOURCEPORT, &fd->stream.para.socket.ip.sourceport)
        >= 0) {
       fd->stream.para.socket.ip.dosourceport = true;
    }
    retropt_bool(opts, OPT_LOWPORT, &fd->stream.para.socket.ip.lowport);
 
-#endif /* WITH_TCP || WITH_UDP */
-
    while (true) {	/* we loop with fork or prohibited packets */
       /* now wait for some packet on this datagram socket, get its sender
 	 address, connect there, and return */
@@ -194,7 +188,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       applyopts(fd->stream.fd, opts, PH_BIND);
       if (Bind(fd->stream.fd, &us.soa, uslen) < 0) {
 	 Error4("bind(%d, {%s}, "F_Zd"): %s", fd->stream.fd,
-		sockaddr_info(&us.soa, infobuff, sizeof(infobuff)),
+		sockaddr_info(&us.soa, uslen, infobuff, sizeof(infobuff)),
 		uslen, strerror(errno));
 	 return STAT_RETRYLATER;
       }
@@ -206,7 +200,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       applyopts(fd->stream.fd, opts, PH_PASTBIND);
 
       Notice1("listening on UDP %s",
-	      sockaddr_info(&us.soa, infobuff, sizeof(infobuff)));
+	      sockaddr_info(&us.soa, uslen, infobuff, sizeof(infobuff)));
       FD_ZERO(&in); FD_ZERO(&out); FD_ZERO(&expt);
       FD_SET(fd->stream.fd, &in);
       while (Select(fd->stream.fd+1, &in, &out, &expt, NULL) < 0) {
@@ -221,13 +215,13 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       if (result < 0) {
 	 Error5("recvfrom(%d, %p, 1, MSG_PEEK, {%s}, {"F_Zu"}): %s",
 		fd->stream.fd, buff1,
-		sockaddr_info(&them->soa, infobuff, sizeof(infobuff)),
+		sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)),
 		themlen, strerror(errno));
 	 return STAT_RETRYLATER;
       }
 
       Notice1("accepting UDP connection from %s",
-	      sockaddr_info(&them->soa, infobuff, sizeof(infobuff)));
+	      sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)));
 
       if (xiocheckpeer(&fd->stream, them, la) < 0) {
 	 /* drop packet */
@@ -236,7 +230,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 	 continue;
       }
       Info1("permitting UDP connection from %s",
-	    sockaddr_info(&them->soa, infobuff, sizeof(infobuff)));
+	    sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)));
 
       if (dofork) {
 	 pid = Fork();
@@ -270,7 +264,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    if ((result = Connect(fd->stream.fd, &them->soa, themlen)) < 0) {
       Error4("connect(%d, {%s}, "F_Zd"): %s",
 	     fd->stream.fd,
-	     sockaddr_info(&them->soa, infobuff, sizeof(infobuff)),
+	     sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)),
 	     themlen, strerror(errno));
       return STAT_RETRYLATER;
    }
@@ -290,11 +284,6 @@ static
 int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
 		     int xioflags, xiofile_t *xxfd, unsigned groups,
 		     int pf, int socktype, int ipproto) {
-   xiosingle_t *xfd = &xxfd->stream;
-   union sockaddr_union them, us;
-   socklen_t themlen = sizeof(them);
-   socklen_t uslen;
-   bool needbind = false;
    int result;
 
    if (argc != 3) {
@@ -302,23 +291,51 @@ int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
 	     argv[0], argc-1);
       return STAT_NORETRY;
    }
+   if ((result = _xioopen_udp_sendto(argv[1], argv[2], opts, xioflags, xxfd,
+				     groups, pf, socktype, ipproto))
+       != STAT_OK) {
+      return result;
+   }
+   _xio_openlate(&xxfd->stream, opts);
+   return STAT_OK;
+}
+
+static
+int _xioopen_udp_sendto(const char *hostname, const char *servname,
+			struct opt *opts,
+		     int xioflags, xiofile_t *xxfd, unsigned groups,
+		     int pf, int socktype, int ipproto) {
+   xiosingle_t *xfd = &xxfd->stream;
+   union sockaddr_union us;
+   socklen_t uslen;
+   int feats = 3;	/* option bind supports address and port */
+   bool needbind = false;
+   int result;
+
+   xfd->howtoend = END_SHUTDOWN;
+   retropt_int(opts, OPT_SO_TYPE, &socktype);
 
    /* ...res_opts[] */
-   applyopts(-1, opts, PH_INIT);
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(-1, opts, PH_INIT);
 
+   xfd->salen = sizeof(xfd->peersa);
    if ((result =
-	xiogetaddrinfo(argv[1], argv[2], pf, socktype, ipproto,
-		       &them, &themlen, xfd->para.socket.ip.res_opts[0],
+	xiogetaddrinfo(hostname, servname, pf, socktype, ipproto,
+		       &xfd->peersa, &xfd->salen,
+		       xfd->para.socket.ip.res_opts[0],
 		       xfd->para.socket.ip.res_opts[1]))
        != STAT_OK) {
       return result;
    }
    if (pf == PF_UNSPEC) {
-      pf = them.soa.sa_family;
+      pf = xfd->peersa.soa.sa_family;
    }
    uslen = socket_init(pf, &us);
-   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, 3) >= 0) {
+   if (retropt_bind(opts, pf, socktype, ipproto, &us.soa, &uslen, feats,
+		    xfd->para.socket.ip.res_opts[0],
+		    xfd->para.socket.ip.res_opts[1])
+       != STAT_NOACTION) {
       needbind = true;
    }
 
@@ -344,29 +361,82 @@ int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
       switch (pf) {
 #if WITH_IP4
       case PF_INET:
+	 /*!!! this is buggy */
 	 us.ip4.sin_port = htons(xfd->para.socket.ip.lowport); break;
 #endif
 #if WITH_IP6
       case PF_INET6: 
+	 /*!!! this is buggy */
 	 us.ip6.sin6_port = htons(xfd->para.socket.ip.lowport); break;
 #endif
       }
       needbind = true;
    }
-   
-   /*!!! lowport */
 
-   /* ...res_opts[] */
-   applyopts(-1, opts, PH_INIT);
-   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
-
-   xfd->dtype = DATA_RECVFROM;
-   return _xioopen_dgram_sendto(&them, themlen, needbind?&us:NULL, uslen,
+   xfd->dtype = XIODATA_RECVFROM;
+   return _xioopen_dgram_sendto(needbind?&us:NULL, uslen,
 			      opts, xioflags, xfd, groups,
 			      pf, socktype, ipproto);
 }
 
 
+static
+int xioopen_udp_datagram(int argc, const char *argv[], struct opt *opts,
+		     int xioflags, xiofile_t *xxfd, unsigned groups,
+		     int pf, int socktype, int ipproto) {
+   xiosingle_t *xfd = &xxfd->stream;
+   char *rangename;
+   char *hostname;
+   int result;
+
+   if (argc != 3) {
+      Error2("%s: wrong number of parameters (%d instead of 2)",
+	     argv[0], argc-1);
+      return STAT_NORETRY;
+   }
+
+   if ((hostname = strdup(argv[1])) == NULL) {
+      Error1("strdup(\"%s\"): out of memory", argv[1]);
+      return STAT_RETRYLATER;
+   }
+
+   result =
+      _xioopen_udp_sendto(hostname, argv[2], opts, xioflags, xxfd, groups,
+			 pf, socktype, ipproto);
+   free(hostname);
+   if (result != STAT_OK) {
+      return result;
+   }
+
+   xfd->dtype = XIOREAD_RECV|XIOWRITE_SENDTO;
+
+   xfd->para.socket.la.soa.sa_family = xfd->peersa.soa.sa_family;
+
+   /* only accept packets with correct remote ports */
+   xfd->para.socket.ip.sourceport = ntohs(xfd->peersa.ip4.sin_port);
+   xfd->para.socket.ip.dosourceport = true;
+
+   /* which reply packets will be accepted - determine by range option */
+   if (retropt_string(opts, OPT_RANGE, &rangename)
+       >= 0) {
+      if (parserange(rangename, pf, &xfd->para.socket.range) < 0) {
+	 free(rangename);
+	 return STAT_NORETRY;
+      }
+      xfd->para.socket.dorange = true;
+      xfd->dtype |= XIOREAD_RECV_CHECKRANGE;
+      free(rangename);
+   }
+
+#if WITH_LIBWRAP
+   xio_retropt_tcpwrap(xfd, opts);
+#endif /* WITH_LIBWRAP */
+
+   _xio_openlate(xfd, opts);
+   return STAT_OK;
+}
+
+
 static
 int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
 		     int xioflags, xiofile_t *xfd, unsigned groups,
@@ -382,6 +452,7 @@ int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
       return STAT_NORETRY;
    }
 
+   xfd->stream.howtoend = END_NONE;
    retropt_socket_pf(opts, &pf);
    if (pf == PF_UNSPEC) {
 #if WITH_IP4 && WITH_IP6
@@ -408,7 +479,10 @@ int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
       union sockaddr_union la;
       socklen_t lalen = sizeof(la);
      
-      if (retropt_bind(opts, pf, socktype, ipproto, &la.soa, &lalen, 1) >= 0) {
+      if (retropt_bind(opts, pf, socktype, ipproto, &la.soa, &lalen, 1,
+		       xfd->stream.para.socket.ip.res_opts[0],
+		       xfd->stream.para.socket.ip.res_opts[1])
+	  != STAT_NOACTION) {
 	 switch (pf) {
 #if WITH_IP4
 	 case PF_INET:  us.ip4.sin_addr  = la.ip4.sin_addr;  break;
@@ -426,9 +500,15 @@ int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
    }
    retropt_bool(opts, OPT_LOWPORT, &xfd->stream.para.socket.ip.lowport);
 
-   xfd->stream.dtype = DATA_RECVFROM_ONE;
-   return _xioopen_dgram_recvfrom(&xfd->stream, xioflags, &us.soa, uslen,
-			      opts, pf, socktype, ipproto, E_ERROR);
+   xfd->stream.dtype = XIODATA_RECVFROM_ONE;
+   if ((result =
+	_xioopen_dgram_recvfrom(&xfd->stream, xioflags, &us.soa, uslen,
+				opts, pf, socktype, ipproto, E_ERROR))
+       != STAT_OK) {
+      return result;
+   }
+   _xio_openlate(&xfd->stream, opts);
+   return STAT_OK;
 }
 
 
@@ -438,6 +518,7 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
 		     int pf, int socktype, int ipproto) {
    union sockaddr_union us;
    socklen_t uslen = sizeof(us);
+   char *rangename;
    int result;
 
    if (argc != 2) {
@@ -474,7 +555,10 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
       socklen_t lalen = sizeof(la);
      
       if (retropt_bind(opts, pf, socktype, ipproto,
-		       &xfd->stream.para.socket.la.soa, &lalen, 1) >= 0) {
+		       &xfd->stream.para.socket.la.soa, &lalen, 1,
+		       xfd->stream.para.socket.ip.res_opts[0],
+		       xfd->stream.para.socket.ip.res_opts[1])
+	  != STAT_NOACTION) {
 	 switch (pf) {
 #if WITH_IP4
 	 case PF_INET:
@@ -492,17 +576,17 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
 #endif
 
 #if WITH_IP4 /*|| WITH_IP6*/
-   if (retropt_string(opts, OPT_RANGE, &xfd->stream.para.socket.rangename) >= 0) {
-      if (parserange(xfd->stream.para.socket.rangename, pf,
-		     &xfd->stream.para.socket.range) < 0) {
+   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
+      if (parserange(rangename, pf, &xfd->stream.para.socket.range) < 0) {
 	 return STAT_NORETRY;
       }
+      xfd->stream.para.socket.dorange = true;
    }
 #endif
 
-#if (WITH_TCP || WITH_UDP) && WITH_LIBWRAP
+#if WITH_LIBWRAP
    xio_retropt_tcpwrap(&xfd->stream, opts);
-#endif /* && (WITH_TCP || WITH_UDP) && WITH_LIBWRAP */
+#endif /* WITH_LIBWRAP */
 
    if (retropt_ushort(opts, OPT_SOURCEPORT,
 		      &xfd->stream.para.socket.ip.sourceport)
@@ -511,9 +595,13 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
    }
    retropt_bool(opts, OPT_LOWPORT, &xfd->stream.para.socket.ip.lowport);
 
-   xfd->stream.dtype = DATA_RECV;
-   result = _xioopen_dgram_recv(&xfd->stream, xioflags, &us.soa, uslen,
-				opts, pf, socktype, ipproto, E_ERROR);
+   xfd->stream.dtype = XIODATA_RECV;
+   if ((result = _xioopen_dgram_recv(&xfd->stream, xioflags, &us.soa, uslen,
+				     opts, pf, socktype, ipproto, E_ERROR))
+       != STAT_OK) {
+      return result;
+   }
+   _xio_openlate(&xfd->stream, opts);
    return result;
 }
 
