@@ -1,5 +1,5 @@
-/* $Id: xioclose.c,v 1.25 2006/05/06 14:20:28 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xioclose.c,v 1.26 2007/01/25 21:36:11 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended close function */
@@ -22,13 +22,13 @@ int xioclose1(struct single *pipe) {
    }
 
 #if WITH_READLINE
-   if (pipe->dtype == DATA_READLINE) {
+   if ((pipe->dtype & XIODATA_MASK) == XIODATA_READLINE) {
       Write_history(pipe->para.readline.history_file);
       /*xiotermios_setflag(pipe->fd, 3, ECHO|ICANON);*/	/* error when pty closed */
    }
 #endif /* WITH_READLINE */
 #if WITH_OPENSSL
-   if (pipe->dtype == DATA_OPENSSL) {
+   if ((pipe->dtype & XIODATA_MASK) == XIODATA_OPENSSL) {
       if (pipe->para.openssl.ssl) {
 	 /* e.g. on TCP connection refused, we do not yet have this set */
 	 sycSSL_shutdown(pipe->para.openssl.ssl);
