@@ -1,5 +1,5 @@
-/* $Id: xioconfig.h,v 1.26 2006/07/13 06:52:44 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xioconfig.h,v 1.27 2007/03/06 21:26:23 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioconfig_h_included
@@ -61,6 +61,27 @@
 #  undef WITH_LIBWRAP
 #endif
 
+#if WITH_SOCKET || WITH_TUN
+#  define _WITH_SOCKET 1
+#endif
+
+#if WITH_IP4 || WITH_TUN
+#  define _WITH_IP4 1
+#endif
+
+#if WITH_IP6 || WITH_TUN
+#  define _WITH_IP6 1
+#endif
+
+#if WITH_NAMED || WITH_TUN
+#  define _WITH_NAMED 1
+#endif
+
+#if WITH_FILE || WITH_TUN
+#  define _WITH_FILE 1
+#endif
+
+
 #if HAVE_DEV_PTMX && HAVE_GRANTPT && HAVE_UNLOCKPT && HAVE_PTSNAME
 #else
 #  undef HAVE_DEV_PTMX
