@@ -1,4 +1,4 @@
-/* $Id: xioinitialize.c,v 1.16 2006/06/30 20:35:21 gerhard Exp $ */
+/* $Id: xioinitialize.c,v 1.17 2006/12/28 14:21:41 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -123,6 +123,40 @@ void xiodroplocks(void) {
    }
 }
 
+
+/* consider an invokation like this:
+   socat -u exec:'some program that accepts data' tcp-l:...,fork
+   we do not want the program to be killed by the first tcp-l sub process, it's
+   better if it survives all sub processes. Thus, it must not be killed if the
+   sub process delivers EOF. Also, a socket that is reused in sub processes
+   should not be shut down (affects the connection), but closed (affects only
+   sub processes copy of file descriptor) */
+static int xio_nokill(xiofile_t *sock) {
+   int result = 0;
+   switch (sock->tag) {
+   case XIO_TAG_INVALID:
+   default:
+      return -1;
+   case XIO_TAG_DUAL:
+      if ((result = xio_nokill((xiofile_t *)sock->dual.stream[0])) != 0)
+	 return result;
+      result = xio_nokill((xiofile_t *)sock->dual.stream[1]);
+      break;
+   case XIO_TAG_RDONLY:
+   case XIO_TAG_WRONLY:
+   case XIO_TAG_RDWR:
+      /* here is the core of this function */
+      switch (sock->stream.howtoend) {
+      case END_SHUTDOWN_KILL: sock->stream.howtoend = END_CLOSE; break;
+      case END_CLOSE_KILL:    sock->stream.howtoend = END_CLOSE; break;
+      case END_SHUTDOWN:      sock->stream.howtoend = END_CLOSE; break;
+      default: break;
+      }
+      break;
+   }
+   return result;
+}
+
 /* call this function immediately after fork() in child process */
 /* it performs some neccessary actions
    returns 0 on success or != 0 if an error occurred */
@@ -140,5 +174,14 @@ int xio_forked_inchild(void) {
 	 Exit(1);
       }
    }
+
+   /* change XIO_SHUTDOWN_KILL to XIO_SHUTDOWN */
+   if (sock1 != NULL) {
+      int result2;
+      result2 = xio_nokill(sock1);
+      if (result2 < 0)  Exit(1);
+      result |= result2;
+   }
+
    return result;
 }
