@@ -1,5 +1,5 @@
-/* $Id: xiomodes.h,v 1.11 2006/07/13 06:53:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xiomodes.h,v 1.13 2007/03/06 21:22:29 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xiomodes_h_included
@@ -21,7 +21,9 @@
 #include "xio-unix.h"
 #include "xio-rawip.h"
 #include "xio-ip.h"
+#if WITH_IP4
 #include "xio-ip4.h"
+#endif /* WITH_IP4 */
 #include "xio-ip6.h"
 #include "xio-ipapp.h"
 #include "xio-tcp.h"
@@ -38,5 +40,6 @@
 #include "xio-openssl.h"
 #include "xio-tcpwrap.h"
 #include "xio-ext2.h"
+#include "xio-tun.h"
 
 #endif /* !defined(__xiomodes_h_included) */
