@@ -1,5 +1,5 @@
-/* $Id: xioopen.c,v 1.110 2006/07/13 06:53:47 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xioopen.c,v 1.119 2007/03/06 21:20:28 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source file of the extended open function */
@@ -22,6 +22,17 @@ const struct addrname addressnames[] = {
 #if WITH_STDIO
    { "-",		&addr_stdio },
 #endif
+#if defined(WITH_UNIX) && defined(WITH_ABSTRACT_UNIXSOCKET)
+   { "abstract",		&xioaddr_abstract_client },
+   { "abstract-client",		&xioaddr_abstract_client },
+   { "abstract-connect",	&xioaddr_abstract_connect },
+#if WITH_LISTEN
+   { "abstract-listen",		&xioaddr_abstract_listen },
+#endif
+   { "abstract-recv",		&xioaddr_abstract_recv },
+   { "abstract-recvfrom",	&xioaddr_abstract_recvfrom },
+   { "abstract-sendto",		&xioaddr_abstract_sendto },
+#endif /* defined(WITH_UNIX) && defined(WITH_ABSTRACT_UNIXSOCKET) */
 #if WITH_CREAT
    { "creat",	&addr_creat },
    { "create",	&addr_creat },
@@ -68,6 +79,8 @@ const struct addrname addressnames[] = {
 #if WITH_RAWIP
 #if (WITH_IP4 || WITH_IP6)
    { "ip",		&addr_rawip_sendto },
+   { "ip-datagram",	&addr_rawip_datagram },
+   { "ip-dgram",	&addr_rawip_datagram },
    { "ip-recv",		&addr_rawip_recv },
    { "ip-recvfrom",	&addr_rawip_recvfrom },
    { "ip-send",	&addr_rawip_sendto },
@@ -75,6 +88,8 @@ const struct addrname addressnames[] = {
 #endif
 #if WITH_IP4
    { "ip4",		&addr_rawip4_sendto },
+   { "ip4-datagram",	&addr_rawip4_datagram },
+   { "ip4-dgram",	&addr_rawip4_datagram },
    { "ip4-recv",	&addr_rawip4_recv },
    { "ip4-recvfrom",	&addr_rawip4_recvfrom },
    { "ip4-send",	&addr_rawip4_sendto },
@@ -82,6 +97,8 @@ const struct addrname addressnames[] = {
 #endif
 #if WITH_IP6
    { "ip6",		&addr_rawip6_sendto },
+   { "ip6-datagram",	&addr_rawip6_datagram },
+   { "ip6-dgram",	&addr_rawip6_datagram },
    { "ip6-recv",	&addr_rawip6_recv },
    { "ip6-recvfrom",	&addr_rawip6_recvfrom },
    { "ip6-send",	&addr_rawip6_sendto },
@@ -160,9 +177,16 @@ const struct addrname addressnames[] = {
    { "tcp6-l",	&addr_tcp6_listen },
    { "tcp6-listen",	&addr_tcp6_listen },
 #endif
+#if WITH_TUN
+   { "tun",		&xioaddr_tun },
+#endif
 #if (WITH_IP4 || WITH_IP6) && WITH_UDP
    { "udp",		&addr_udp_connect },
+#endif
+#if (WITH_IP4 || WITH_IP6) && WITH_UDP
    { "udp-connect",	&addr_udp_connect },
+   { "udp-datagram",	&addr_udp_datagram },
+   { "udp-dgram",	&addr_udp_datagram },
 #endif
 #if (WITH_IP4 || WITH_IP6) && WITH_UDP && WITH_LISTEN
    { "udp-l",	&addr_udp_listen },
@@ -177,6 +201,8 @@ const struct addrname addressnames[] = {
 #if WITH_IP4 && WITH_UDP
    { "udp4",		&addr_udp4_connect },
    { "udp4-connect",	&addr_udp4_connect },
+   { "udp4-datagram",	&addr_udp4_datagram },
+   { "udp4-dgram",	&addr_udp4_datagram },
 #endif
 #if WITH_IP4 && WITH_UDP && WITH_LISTEN
    { "udp4-l",		&addr_udp4_listen },
@@ -191,9 +217,11 @@ const struct addrname addressnames[] = {
 #if WITH_IP6 && WITH_UDP
    { "udp6",		&addr_udp6_connect },
    { "udp6-connect",	&addr_udp6_connect },
+   { "udp6-datagram",	&addr_udp6_datagram },
+   { "udp6-dgram",	&addr_udp6_datagram },
 #endif
 #if WITH_IP6 && WITH_UDP && WITH_LISTEN
-   { "udp6-l",	&addr_udp6_listen },
+   { "udp6-l",		&addr_udp6_listen },
    { "udp6-listen",	&addr_udp6_listen },
 #endif
 #if WITH_IP6 && WITH_UDP
@@ -267,11 +295,11 @@ static xiofile_t *xioallocfd(void) {
 /* fd->common.eof       = 0; */
 
    fd->stream.fd        = -1;
-   fd->stream.dtype     = DATA_STREAM;
+   fd->stream.dtype     = XIODATA_STREAM;
 #if WITH_SOCKET
 /* fd->stream.salen     = 0; */
 #endif /* WITH_SOCKET */
-   fd->stream.howtoend  = END_CLOSE;
+   fd->stream.howtoend  = END_UNSPEC;
 /* fd->stream.name      = NULL; */
 /* fd->stream.para.exec.pid = 0; */
    fd->stream.lineterm  = LINETERM_RAW;
@@ -449,8 +477,8 @@ static xiosingle_t *xioparse_single(const char **addr) {
 	    Error1("strdup(\"%s\"): out of memory", token);
 	 }
 	 /*! check argc overflow */
-      } else {
 #endif /* WITH_GOPEN */
+      } else {
 	 Error1("unknown device/address \"%s\"", token);
 	 /*!!! free something*/ return NULL;
       }
