@@ -1,5 +1,5 @@
-/* $Id: xioopts.h,v 1.64 2006/07/08 10:03:44 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xioopts.h,v 1.70 2007/03/06 21:19:46 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioopts_h_included
@@ -49,6 +49,8 @@ enum e_types {
    TYPE_LONGLONG,	/* long long */
    TYPE_OFF32,		/* off_t */
    TYPE_OFF64,		/* off64_t */
+   TYPE_IP_MREQN,	/* for  struct ip_mreq  or  struct ip_mreqn */
+   TYPE_IP4NAME,	/* IPv4 hostname or address */
 
    TYPE_2BYTE = TYPE_USHORT
 } ;
@@ -81,7 +83,8 @@ enum e_func {
    OFUNC_TERMIOS_SPEED,	/* termios c_ispeed etc on FreeBSD */
    OFUNC_TERMIOS_SPEC,	/* termios combined modes */
    OFUNC_SIGNAL,	/* a signal that should be passed to child process */
-   OFUNC_RESOLVER	/* a bit position used on _res.options */
+   OFUNC_RESOLVER,	/* a bit position used on _res.options */
+   OFUNC_IFFLAG		/* interface flag: locical-or a 1bit mask */
 } ;
 
 /* for simpler handling of option-to-connection-type relations we define
@@ -130,6 +133,8 @@ enum e_func {
 #define GROUP_SOCK_IP4	0x00200000
 #define GROUP_SOCK_IP6	0x00400000
 #define GROUP_SOCK_IP	(GROUP_SOCK_IP4|GROUP_SOCK_IP6)
+#define GROUP_INTERFACE	0x00800000
+#define GROUP_TUN       GROUP_INTERFACE
 
 #define GROUP_IP_UDP	0x01000000
 #define GROUP_IP_TCP	0x02000000
@@ -242,6 +247,7 @@ enum e_optcode {
 #ifdef ECHOPRT
    OPT_ECHOPRT,		/* termios.c_lflag */
 #endif
+   OPT_END_CLOSE,	/* xfd.stream.howtoend = END_CLOSE */
    OPT_EXT2_SECRM,
    OPT_EXT2_UNRM,
    OPT_EXT2_COMPR,
@@ -278,8 +284,10 @@ enum e_optcode {
    OPT_FORK,
    OPT_FTRUNCATE32,	/* ftruncate() */
    OPT_FTRUNCATE64,	/* ftruncate64() */
-   OPT_F_SETLK,		/* fcntl with struct flock */
-   OPT_F_SETLKW,	/* fcntl with struct flock */
+   OPT_F_SETLKW_RD,	/* fcntl with struct flock - read-lock, wait */
+   OPT_F_SETLKW_WR,	/* fcntl with struct flock - write-lock, wait */
+   OPT_F_SETLK_RD,	/* fcntl with struct flock - read-lock */
+   OPT_F_SETLK_WR,	/* fcntl with struct flock - write-lock */
    OPT_GROUP,
    OPT_GROUP_EARLY,
    OPT_GROUP_LATE,
@@ -288,6 +296,23 @@ enum e_optcode {
    OPT_ICANON,		/* termios.c_lflag */
    OPT_ICRNL,		/* termios.c_iflag */
    OPT_IEXTEN,		/* termios.c_lflag */
+   OPT_IFF_ALLMULTI,	/* struct ifreq.ifr_flags */
+   OPT_IFF_AUTOMEDIA,	/* struct ifreq.ifr_flags */
+   OPT_IFF_BROADCAST,	/* struct ifreq.ifr_flags */
+   OPT_IFF_DEBUG,	/* struct ifreq.ifr_flags */
+   /*OPT_IFF_DYNAMIC,*/	/* struct ifreq.ifr_flags */
+   OPT_IFF_LOOPBACK,	/* struct ifreq.ifr_flags */
+   OPT_IFF_MASTER,	/* struct ifreq.ifr_flags */
+   OPT_IFF_MULTICAST,	/* struct ifreq.ifr_flags */
+   OPT_IFF_NOARP,	/* struct ifreq.ifr_flags */
+   OPT_IFF_NOTRAILERS,	/* struct ifreq.ifr_flags */
+   OPT_IFF_NO_PI,	/* tun: IFF_NO_PI */
+   OPT_IFF_PORTSEL,	/* struct ifreq.ifr_flags */
+   OPT_IFF_POINTOPOINT,	/* struct ifreq.ifr_flags */
+   OPT_IFF_PROMISC,	/* struct ifreq.ifr_flags */
+   OPT_IFF_RUNNING,	/* struct ifreq.ifr_flags */
+   OPT_IFF_SLAVE,	/* struct ifreq.ifr_flags */
+   OPT_IFF_UP,		/* struct ifreq.ifr_flags */
    OPT_IGNBRK,		/* termios.c_iflag */
    OPT_IGNCR,		/* termios.c_iflag */
    OPT_IGNORECR,	/* HTTP */
@@ -297,10 +322,12 @@ enum e_optcode {
    OPT_INLCR,		/* termios.c_iflag */
    OPT_INPCK,		/* termios.c_iflag */
    OPT_INTERVALL,
+   OPT_IPV6_JOIN_GROUP,
    OPT_IPV6_V6ONLY,
 #if 0	/* see Linux: man 7 netlink; probably not what we need yet */
    OPT_IO_SIOCGIFNAME,
 #endif
+   OPT_IP_ADD_MEMBERSHIP,
 #ifdef IP_HDRINCL
    OPT_IP_HDRINCL,
 #endif
@@ -313,6 +340,7 @@ enum e_optcode {
 #ifdef IP_MTU_DISCOVER
    OPT_IP_MTU_DISCOVER,
 #endif
+   OPT_IP_MULTICAST_IF,
    OPT_IP_MULTICAST_LOOP,
    OPT_IP_MULTICAST_TTL,
    OPT_IP_OPTIONS,
@@ -686,7 +714,11 @@ enum e_optcode {
 #endif
    OPT_TIOCSCTTY,
    OPT_TOSTOP,		/* termios.c_lflag */
+   OPT_TUN_DEVICE,	/* tun: /dev/net/tun ... */
+   OPT_TUN_NAME,	/* tun: tun0 */
+   OPT_TUN_TYPE,	/* tun: tun|tap */
    OPT_UMASK,
+   OPT_UNIX_TIGHTSOCKLEN,	/* UNIX domain sockets */
    OPT_UNLINK,
    OPT_UNLINK_CLOSE,
    OPT_UNLINK_EARLY,
@@ -810,9 +842,9 @@ extern int retropt_bind(struct opt *opts,
 		 int ipproto,
 		 struct sockaddr *sa,
 		 socklen_t *salen,
-		 int feats	/* TCP etc: 1..address allowed,
+		 int feats,	/* TCP etc: 1..address allowed,
 				   3..address and port allowed */
-		 );
+		 unsigned long res_opts0, unsigned long res_opts1);
 extern int applyopts(int fd, struct opt *opts, unsigned int phase);
 extern int applyopts2(int fd, struct opt *opts, unsigned int from,
 		      unsigned int to);
@@ -825,6 +857,9 @@ extern int applyopts_offset(struct single *xfd, struct opt *opts);
 extern int applyopts_signal(struct single *xfd, struct opt *opts);
 extern int _xio_openlate(struct single *fd, struct opt *opts);
 extern int parseopts(const char **a, unsigned int groups, struct opt **opts);
+extern int parseopts_table(const char **a, unsigned int groups,
+			   struct opt **opts,
+			 const struct optname optionnames[], size_t optionnum);
 extern struct opt *copyopts(const struct opt *opts, unsigned int groups);
 extern struct opt *moveopts(struct opt *opts, unsigned int groups);
 extern int leftopts(const struct opt *opts);
