@@ -1,5 +1,5 @@
-/* $Id: xioread.c,v 1.32 2006/07/13 18:19:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xioread.c,v 1.38 2007/03/06 21:20:07 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended read function */
@@ -24,7 +24,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
    int _errno;
 
    if (file->tag == XIO_TAG_INVALID) {
-      Error("xioread(): invalid file descriptor");
+      Error1("xioread(): invalid xiofile descriptor %p", file);
       errno = EINVAL;
       return -1;
    }
@@ -32,7 +32,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
    if (file->tag == XIO_TAG_DUAL) {
       pipe = file->dual.stream[0];
       if (pipe->tag == XIO_TAG_INVALID) {
-	 Error("xioread(): invalid file descriptor");
+	 Error1("xioread(): invalid xiofile sub descriptor %p[0]", file);
 	 errno = EINVAL;
 	 return -1;
       }
@@ -50,8 +50,8 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
       }
    }
 
-   if (pipe->dtype == DATA_STREAM || pipe->dtype == DATA_PIPE ||
-       pipe->dtype == DATA_2PIPE) {
+   switch (pipe->dtype & XIODATA_READMASK) {
+   case XIOREAD_STREAM:
       do {
 	 bytes = Read(pipe->fd, buff, bufsiz);
       } while (bytes < 0 && errno == EINTR);
@@ -71,7 +71,9 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 errno = _errno;
 	 return -1;
       }
-   } else if (pipe->dtype == DATA_PTY) {
+      break;
+
+   case XIOREAD_PTY:
       do {
 	 bytes = Read(pipe->fd, buff, bufsiz);
       } while (bytes < 0 && errno == EINTR);
@@ -88,23 +90,28 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 errno = _errno;
 	 return -1;
       }
+      break;
+
 #if WITH_READLINE
-   } else if (pipe->dtype == DATA_READLINE) {
+   case XIOREAD_READLINE:
       if ((bytes = xioread_readline(pipe, buff, bufsiz)) < 0) {
 	 return -1;
       }      
+      break;
 #endif /* WITH_READLINE */
+
 #if WITH_OPENSSL
-   } else if (pipe->dtype == DATA_OPENSSL) {
+   case XIOREAD_OPENSSL:
       /* this function prints its error messages */
       if ((bytes = xioread_openssl(pipe, buff, bufsiz)) < 0) {
 	 return -1;
       }
+      break;
 #endif /* WITH_OPENSSL */
-   } else if (pipe->dtype == DATA_RECVFROM ||
-	      pipe->dtype == DATA_RECVFROM_SKIPIP ||
-	      pipe->dtype == DATA_RECVFROM_ONE ||
-	      pipe->dtype == DATA_RECVFROM_SKIPIP_ONE) {
+
+#if WITH_SOCKET
+   case XIOREAD_RECV:
+     if (pipe->dtype & XIOREAD_RECV_FROM) {
 #if WITH_RAWIP || WITH_UDP || WITH_UNIX
       union sockaddr_union from = {{0}};
       socklen_t fromlen = sizeof(from);
@@ -119,19 +126,20 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 _errno = errno;
 	 Error6("recvfrom(%d, %p, "F_Zu", 0, %s, {"F_socklen"}): %s",
 		pipe->fd, buff, bufsiz,
-		sockaddr_info(&from.soa, infobuff, sizeof(infobuff)),
+		sockaddr_info(&from.soa, fromlen, infobuff, sizeof(infobuff)),
 		fromlen, strerror(errno));
 	 errno = _errno;
 	 return -1;
       }
+      Notice2("received packet with "F_Zu" bytes from %s",
+	      bytes,
+	      sockaddr_info(&from.soa, fromlen, infobuff, sizeof(infobuff)));
       if (bytes == 0) {
 	 if (!pipe->para.socket.emptyiseof) {
 	    errno = EAGAIN; return -1;
 	 }
 	 return bytes;
       }
-      Notice2("received packet with "F_Zu" bytes from %s",
-	      bytes, sockaddr_info(&from.soa, infobuff, sizeof(infobuff)));
 
       if (pipe->peersa.soa.sa_family != PF_UNSPEC) {
 	 /* a peer address is defined, so we need to check if it matches */
@@ -141,9 +149,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	    errno = EAGAIN; return -1;
 	 }
 #endif
-	 switch (pipe->dtype) {
-	 case DATA_RECVFROM_SKIPIP/*!*/:
-	 case DATA_RECVFROM_SKIPIP_ONE/*!*/:
+	 if (pipe->dtype & XIOREAD_RECV_SKIPIP) {
 	    if (pipe->peersa.soa.sa_family != from.soa.sa_family) {
 	       Info("recvfrom(): wrong peer protocol, ignoring packet");
 	       errno = EAGAIN; return -1;
@@ -159,8 +165,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	       break;
 	    }
 #endif /* WITH_IP4 */
-	    break;
-	 default:
+	 } else {
 	    switch (pipe->peersa.soa.sa_family) {
 #if 0
 	    case PF_UNIX:
@@ -195,8 +200,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 int headlen;
 #if WITH_IP4
       case AF_INET:
-	 if (pipe->dtype == DATA_RECVFROM_SKIPIP ||
-	     pipe->dtype == DATA_RECVFROM_SKIPIP_ONE) {
+	 if (pipe->dtype & XIOREAD_RECV_SKIPIP) {
 	    /* IP4 raw sockets include the header when passing a packet to the
 	       application - we don't need it here. */
 #if HAVE_STRUCT_IP_IP_HL
@@ -224,7 +228,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 /* do nothing, for now */
 	 break;
       }
-      if (pipe->dtype = DATA_RECVFROM_ONE || pipe->dtype == DATA_RECVFROM_SKIPIP_ONE) {
+      if (pipe->dtype & XIOREAD_RECV_ONESHOT) {
 #if 1
 	 pipe->eof = 2;
 #else
@@ -276,9 +280,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
       return -1;
 #endif /* !WITH_RAWIP || WITH_UDP || WITH_UNIX */
 
-#if WITH_SOCKET
-   } else if (pipe->dtype == DATA_RECV ||
-	      pipe->dtype == DATA_RECV_SKIPIP) {
+     } else /* ~XIOREAD_RECV_FROM */ {
       union sockaddr_union from;  socklen_t fromlen = sizeof(from);
       char infobuff[256];
 
@@ -292,8 +294,9 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 errno = EAGAIN;  return -1;
       }
       Info1("permitting packet from %s",
-	    sockaddr_info((struct sockaddr *)&from,
+	    sockaddr_info((struct sockaddr *)&from, fromlen,
 			  infobuff, sizeof(infobuff)));
+
       do {
 	 bytes =
 	    Recvfrom(pipe->fd, buff, bufsiz, 0, &from.soa, &fromlen);
@@ -303,11 +306,14 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 _errno = errno;
 	 Error6("recvfrom(%d, %p, "F_Zu", 0, %s, "F_socklen"): %s",
 		pipe->fd, buff, bufsiz,
-		sockaddr_info(&from.soa, infobuff, sizeof(infobuff)),
+		sockaddr_info(&from.soa, fromlen, infobuff, sizeof(infobuff)),
 		fromlen, strerror(errno));
 	 errno = _errno;
 	 return -1;
       }
+      Notice2("received packet with "F_Zu" bytes from %s",
+	      bytes,
+	      sockaddr_info(&from.soa, fromlen, infobuff, sizeof(infobuff)));
       if (bytes == 0) {
 	 if (!pipe->para.socket.emptyiseof) {
 	    errno = EAGAIN; return -1;
@@ -319,7 +325,7 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 int headlen;
 #if WITH_IP4
       case AF_INET:
-	 if (pipe->dtype == DATA_RECV_SKIPIP) {
+	 if (pipe->dtype & XIOREAD_RECV_SKIPIP) {
 	    /* IP4 raw sockets include the header when passing a packet to the
 	       application - we don't need it here. */
 #if HAVE_STRUCT_IP_IP_HL
@@ -346,14 +352,50 @@ ssize_t xioread(xiofile_t *file, void *buff, size_t bufsiz) {
 	 break;
       }
 
-      Notice2("received packet with "F_Zu" bytes from %s",
-	      bytes, sockaddr_info(&from.soa, infobuff, sizeof(infobuff)));
+     }
+     break;
 #endif /* WITH_SOCKET */
 
-   } else {
+   default:
       Error("internal: undefined read operation");
       errno = EINVAL;  return -1;
    }
    pipe->actbytes -= bytes;
    return bytes;
 }
+
+
+/* this function is intended only for some special address types where the
+   select() call cannot strictly determine if (more) read data is available.
+   currently this is for the OpenSSL based addresses.
+*/
+ssize_t xiopending(xiofile_t *file) {
+   struct single *pipe;
+
+   if (file->tag == XIO_TAG_INVALID) {
+      Error1("xiopending(): invalid xiofile descriptor %p", file);
+      errno = EINVAL;
+      return -1;
+   }
+
+   if (file->tag == XIO_TAG_DUAL) {
+      pipe = file->dual.stream[0];
+      if (pipe->tag == XIO_TAG_INVALID) {
+	 Error1("xiopending(): invalid xiofile sub descriptor %p[0]", file);
+	 errno = EINVAL;
+	 return -1;
+      }
+   } else {
+      pipe = &file->stream;
+   }
+
+   switch (pipe->dtype & XIODATA_READMASK) {
+#if WITH_OPENSSL
+   case XIOREAD_OPENSSL:
+      return xiopending_openssl(pipe);
+#endif /* WITH_OPENSSL */
+   default:
+      return 0;
+   }
+}
+
