@@ -1,5 +1,5 @@
-/* $Id: xiowrite.c,v 1.25 2006/07/08 10:03:47 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* $Id: xiowrite.c,v 1.29 2007/02/08 18:27:00 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2007 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended write function */
@@ -13,6 +13,9 @@
 
 
 /* ...
+   note that the write() call can block even if the select() call reported the
+   FD writeable: in case the FD is not nonblocking and a lock defers the
+   operation.
    on return value < 0: errno reflects the value from write() */
 ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
    ssize_t writt;
@@ -20,7 +23,7 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
    int _errno;
 
    if (file->tag == XIO_TAG_INVALID) {
-      Error("xiowrite(): invalid file descriptor");
+      Error1("xiowrite(): invalid xiofile descriptor %p", file);
       errno = EINVAL;
       return -1;
    }
@@ -28,7 +31,7 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
    if (file->tag == XIO_TAG_DUAL) {
       pipe = file->dual.stream[1];
       if (pipe->tag == XIO_TAG_INVALID) {
-	 Error("xiowrite(): invalid file descriptor");
+	 Error1("xiowrite(): invalid xiofile sub descriptor %p[1]", file);
 	 errno = EINVAL;
 	 return -1;
       }
@@ -38,13 +41,14 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 
 #if WITH_READLINE
    /* try to extract a prompt from the write data */
-   if (pipe->dtype == DATA_READLINE) {
+   if ((pipe->dtype & XIODATA_READMASK) == XIOREAD_READLINE) {
       xioscan_readline(pipe, buff, bytes);
    }
 #endif /* WITH_READLINE */
 
-   if (pipe->dtype == DATA_STREAM || pipe->dtype == DATA_READLINE ||
-       pipe->dtype == DATA_PTY) {
+   switch (pipe->dtype & XIODATA_WRITEMASK) {
+
+   case XIOWRITE_STREAM:
       do {
 	 writt = Write(pipe->fd, buff, bytes);
       } while (writt < 0 && errno == EINTR);
@@ -70,8 +74,10 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 Warn2("write() only wrote "F_Zu" of "F_Zu" bytes",
 	       writt, bytes);
       }
+      break;
+
 #if WITH_SOCKET
-   } else if (pipe->dtype == DATA_RECVFROM || pipe->dtype == DATA_RECVFROM_SKIPIP) {
+   case XIOWRITE_SENDTO:
       /*union {
 	 char space[sizeof(struct sockaddr_un)];
 	 struct sockaddr sa;
@@ -87,9 +93,9 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 _errno = errno;
 	 Error6("sendto(%d, %p, "F_Zu", 0, %s, "F_socklen"): %s",
 		pipe->fd, buff, bytes, 
-		 sockaddr_info(&pipe->peersa.soa,
-			       infobuff, sizeof(infobuff)),
-		 pipe->salen, strerror(_errno));
+		sockaddr_info(&pipe->peersa.soa, pipe->salen,
+			      infobuff, sizeof(infobuff)),
+		pipe->salen, strerror(_errno));
 	 errno = _errno;
 	 return -1;
       }
@@ -97,7 +103,7 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 char infobuff[256];
 	 Warn7("sendto(%d, %p, "F_Zu", 0, %s, "F_socklen") only wrote "F_Zu" of "F_Zu" bytes",
 	       pipe->fd, buff, bytes, 
-	       sockaddr_info(&pipe->peersa.soa,
+	       sockaddr_info(&pipe->peersa.soa, pipe->salen,
 			     infobuff, sizeof(infobuff)),
 	       pipe->salen, writt, bytes);
       } else {
@@ -108,10 +114,12 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 socklen_t uslen = sizeof(us);
 	 Getsockname(pipe->fd, &us.soa, &uslen);
 	 Notice1("local address: %s",
-		 sockaddr_info(&us.soa, infobuff, sizeof(infobuff)));
+		 sockaddr_info(&us.soa, uslen, infobuff, sizeof(infobuff)));
       }
+      break;
 #endif /* WITH_SOCKET */
-   } else if (pipe->dtype == DATA_PIPE) {
+
+   case XIOWRITE_PIPE:
       do {
 	 writt = Write(pipe->para.bipipe.fdout, buff, bytes);
       } while (writt < 0 && errno == EINTR);
@@ -126,7 +134,9 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 Warn2("write() only wrote "F_Zu" of "F_Zu" bytes",
 	       writt, bytes);
       }
-   } else if (pipe->dtype == DATA_2PIPE) {
+      break;
+
+   case XIOWRITE_2PIPE:
       do {
 	 writt = Write(pipe->para.exec.fdout, buff, bytes);
       } while (writt < 0 && errno == EINTR);
@@ -141,12 +151,15 @@ ssize_t xiowrite(xiofile_t *file, const void *buff, size_t bytes) {
 	 Warn2("write() only processed "F_Zu" of "F_Zu" bytes",
 	       writt, bytes);
       }
+      break;
+
 #if WITH_OPENSSL
-   } else if (pipe->dtype == DATA_OPENSSL) {
+   case XIOWRITE_OPENSSL:
       /* this function prints its own error messages */
       return xiowrite_openssl(pipe, buff, bytes);
 #endif /* WITH_OPENSSL */
-   } else {
+
+   default:
       Error1("xiowrite(): bad data type specification %d", pipe->dtype);
       errno = EINVAL;
       return -1;
